
from urllib import addinfourl

class URLError(IOError): ...

class HTTPError(URLError, addinfourl): ...

class Request(object):
    host = ... # type: str
    port = ... # type: str
    data = ... # type: str
    headers = ... # type: Dict[str, str]
    unverifiable = ... # type: bool
    type = ...
    origin_req_host= ...
    unredirected_hdrs = ...

    def __init__(self, url: str, data: str = None, headers: Dict[str, str] = ...,
                 origin_req_host: str = None, unverifiable: bool = ...) -> None: ...
    def __getattr__(self, attr): ...
    def get_method(self) -> str: ...
    def add_data(self, data) -> None: ...
    def has_data(self) -> bool: ...
    def get_data(self) -> str: ...
    def get_full_url(self) -> str: ...
    def get_type(self): ...
    def get_host(self) -> str: ...
    def get_selector(self): ...
    def set_proxy(self, host, type)-> None: ...
    def has_proxy(self) -> bool: ...
    def get_origin_req_host(self) -> str: ...
    def is_unverifiable(self) -> bool: ...
    def add_header(self, key: str, val: str) -> None: ...
    def add_unredirected_header(self, key: str, val: str) -> None: ...
    def has_header(self, header_name: str) -> bool: ...
    def get_header(self, header_name: str, default: str = None) -> str: ...
    def header_items(self): ...

class OpenerDirector(object): ...

def urlopen(url, data=None, timeout=..., cafile=None, capath=None, cadefault=False,
            context=None): ...
def install_opener(opener): ...
def build_opener(*handlers): ...

class BaseHandler:
    handler_order = ... # int

    def add_parent(self, parent) -> None: ...
    def close(self) -> None: ...
    def __lt__(self, other) -> bool: ...

class HTTPErrorProcessor(BaseHandler):
    def http_response(self, request, response): ...

class HTTPDefaultErrorHandler(BaseHandler):
    def http_error_default(self, req, fp, code, msg, hdrs): ...

class HTTPRedirectHandler(BaseHandler):
    max_repeats = ... # int
    max_redirections = ... # int
    def redirect_request(self, req, fp, code, msg, headers, newurl): ...
    def http_error_302(self, req, fp, code, msg, headers): ...
    http_error_301 = http_error_303 = http_error_307 = http_error_302
    inf_msg = ... # type: str


class ProxyHandler(BaseHandler):
    def __init__(self, proxies=None): ...
    def proxy_open(self, req, proxy, type): ...

class HTTPPasswordMgr:
    def __init__(self) -> None: ...
    def add_password(self, realm, uri, user, passwd): ...
    def find_user_password(self, realm, authuri): ...
    def reduce_uri(self, uri, default_port: bool): ...
    def is_suburi(self, base, test): ...

class HTTPPasswordMgrWithDefaultRealm(HTTPPasswordMgr): ...

class AbstractBasicAuthHandler:
    def __init__(self, password_mgr=None): ...
    def reset_retry_count(self): ...
    def http_error_auth_reqed(self, authreq, host, req, headers): ...
    def retry_http_basic_auth(self, host, req, realm): ...

class HTTPBasicAuthHandler(AbstractBasicAuthHandler, BaseHandler):
    auth_header = ... # type: str
    def http_error_401(self, req, fp, code, msg, headers): ...

class ProxyBasicAuthHandler(AbstractBasicAuthHandler, BaseHandler):
    auth_header = ... # type: str
    def http_error_407(self, req, fp, code, msg, headers): ...

class AbstractDigestAuthHandler:
    def __init__(self, passwd=None): ...
    def reset_retry_count(self): ...
    def http_error_auth_reqed(self, auth_header, host, req, headers): ...
    def retry_http_digest_auth(self, req, auth): ...
    def get_cnonce(self, nonce): ...
    def get_authorization(self, req, chal): ...
    def get_algorithm_impls(self, algorithm): ...
    def get_entity_digest(self, data, chal): ...

class HTTPDigestAuthHandler(BaseHandler, AbstractDigestAuthHandler):
    auth_header = ... # str
    handler_order = ... # int
    def http_error_401(self, req, fp, code, msg, headers): ...

class ProxyDigestAuthHandler(BaseHandler, AbstractDigestAuthHandler):
    auth_header = ... # str
    handler_order = ... # int
    def http_error_407(self, req, fp, code, msg, headers): ...

class AbstractHTTPHandler(BaseHandler):
    def __init__(self, debuglevel: int) -> None: ...
    def do_request_(self, request): ...
    def do_open(self, http_class, req): ...

class HTTPHandler(AbstractHTTPHandler):
    def http_open(self, req): ...
    http_request = AbstractHTTPHandler.do_request_

class HTTPSHandler(AbstractHTTPHandler):
    def __init__(self, debuglevel=0, context=None): ...
    def https_open(self, req): ...
    https_request = AbstractHTTPHandler.do_request_

class HTTPCookieProcessor(BaseHandler):
    def __init__(self, cookiejar=None): ...
    def http_request(self, request): ...
    def http_response(self, request, response): ...

class UnknownHandler(BaseHandler):
    def unknown_open(self, req): ...

class FileHandler(BaseHandler):
    def file_open(self, req): ...
    def get_names(self): ...
    def open_local_file(self, req): ...

class FTPHandler(BaseHandler):
    def ftp_open(self, req): ...
    def connect_ftp(self, user, passwd, host, port, dirs, timeout): ...

class CacheFTPHandler(FTPHandler):
    def __init__(self): ...
    def setTimeout(self, t): ...
    def setMaxConns(self, m): ...
    def check_cache(self): ...
    def clear_cache(self): ...



# Stubs for packaging.specifiers (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.
import abc

class InvalidSpecifier(ValueError): ...

class BaseSpecifier:
    def __hash__(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    @abc.abstractproperty
    def prereleases(self): ...
    @prereleases.setter
    def prereleases(self, value): ...
    def contains(self, item, prereleases=None): ...
    def filter(self, iterable, prereleases=None): ...

class _IndividualSpecifier(BaseSpecifier):
    def __init__(self, spec:str=..., prereleases=None) -> None: ...
    def __hash__(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    @property
    def operator(self): ...
    @property
    def version(self): ...
    @property
    def prereleases(self): ...
    @prereleases.setter
    def prereleases(self, value): ...
    def __contains__(self, item): ...
    def contains(self, item, prereleases=None): ...
    def filter(self, iterable, prereleases=None): ...

class LegacySpecifier(_IndividualSpecifier): ...

class Specifier(_IndividualSpecifier):
    @property
    def prereleases(self): ...
    @prereleases.setter
    def prereleases(self, value): ...

class SpecifierSet(BaseSpecifier):
    def __init__(self, specifiers:str=..., prereleases=None) -> None: ...
    def __hash__(self): ...
    def __and__(self, other): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __len__(self): ...
    def __iter__(self): ...
    @property
    def prereleases(self): ...
    @prereleases.setter
    def prereleases(self, value): ...
    def __contains__(self, item): ...
    def contains(self, item, prereleases=None): ...
    def filter(self, iterable, prereleases=None): ...



# Stubs for warnings

# Based on http://docs.python.org/3.2/library/warnings.html

from typing import Any, List, TextIO, Union

def warn(message: Union[str, Warning], category: type = ...,
         stacklevel: int = ...) -> None: ...

def warn_explicit(message: Union[str, Warning], category: type, filename: str,
                  lineno: int, module: str = ..., registry: Any = ...,
                  module_globals: Any = ...) -> None: ...

# logging modifies showwarning => make it a variable.
def _showwarning(message: str, category: type, filename: str, lineno: int,
                 file: TextIO = ..., line: str = ...) -> None: ...
showwarning = _showwarning

def formatwarning(message: str, category: type, filename: str, lineno: int,
                  line: str = ...) -> None: ...
def filterwarnings(action: str, message: str = ..., category: type = ...,
                   module: str = ..., lineno: int = ...,
                   append: bool = ...) -> None: ...
def simplefilter(action: str, category: type = ..., lineno: int = ...,
                 append: bool = ...) -> None: ...
def resetwarnings() -> None: ...

class catch_warnings:
    # TODO record and module must be keyword arguments!
    # TODO type of module?
    def __init__(self, record: bool = ..., module: Any = ...) -> None: ...
    def __enter__(self) -> List[Any]: ...
    def __exit__(self, type, value, traceback) -> bool: ...



# Stubs for requests.auth (Python 3)

from typing import Any
from . import compat
from . import cookies
from . import utils
from . import status_codes

extract_cookies_to_jar = cookies.extract_cookies_to_jar
parse_dict_header = utils.parse_dict_header
to_native_string = utils.to_native_string
codes = status_codes.codes

CONTENT_TYPE_FORM_URLENCODED = ...  # type: Any
CONTENT_TYPE_MULTI_PART = ...  # type: Any

class AuthBase:
    def __call__(self, r): ...

class HTTPBasicAuth(AuthBase):
    username = ...  # type: Any
    password = ...  # type: Any
    def __init__(self, username, password) -> None: ...
    def __call__(self, r): ...

class HTTPProxyAuth(HTTPBasicAuth):
    def __call__(self, r): ...

class HTTPDigestAuth(AuthBase):
    username = ...  # type: Any
    password = ...  # type: Any
    last_nonce = ...  # type: Any
    nonce_count = ...  # type: Any
    chal = ...  # type: Any
    pos = ...  # type: Any
    num_401_calls = ...  # type: Any
    def __init__(self, username, password) -> None: ...
    def build_digest_header(self, method, url): ...
    def handle_redirect(self, r, **kwargs): ...
    def handle_401(self, r, **kwargs): ...
    def __call__(self, r): ...



# Stubs for sqlalchemy.orm (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
## from . import mapper
## from . import interfaces
## from . import deprecated_interfaces
## from . import util
## from . import properties
## from . import relationships
## from . import descriptor_props
from . import session
## from . import scoping
## from . import query
from ..util import langhelpers
## from . import strategy_options

## Mapper = mapper.Mapper
## class_mapper = mapper.class_mapper
## configure_mappers = mapper.configure_mappers
## reconstructor = mapper.reconstructor
## validates = mapper.validates
## EXT_CONTINUE = interfaces.EXT_CONTINUE
## EXT_STOP = interfaces.EXT_STOP
## PropComparator = interfaces.PropComparator
## MapperExtension = deprecated_interfaces.MapperExtension
## SessionExtension = deprecated_interfaces.SessionExtension
## AttributeExtension = deprecated_interfaces.AttributeExtension
## aliased = util.aliased
## join = util.join
## object_mapper = util.object_mapper
## outerjoin = util.outerjoin
## polymorphic_union = util.polymorphic_union
## was_deleted = util.was_deleted
## with_parent = util.with_parent
## with_polymorphic = util.with_polymorphic
## ColumnProperty = properties.ColumnProperty
## RelationshipProperty = relationships.RelationshipProperty
## ComparableProperty = descriptor_props.ComparableProperty
## CompositeProperty = descriptor_props.CompositeProperty
## SynonymProperty = descriptor_props.SynonymProperty
## foreign = relationships.foreign
## remote = relationships.remote
Session = session.Session
object_session = Session.object_session
sessionmaker = session.sessionmaker
## make_transient = session.make_transient
## make_transient_to_detached = session.make_transient_to_detached
## scoped_session = scoping.scoped_session
## AliasOption = query.AliasOption
## Query = query.Query
## Bundle = query.Bundle
public_factory = langhelpers.public_factory

def create_session(bind=..., **kwargs): ...

relationship = ... # type: Any

def relation(*arg, **kw): ...
def dynamic_loader(argument, **kw): ...

column_property = ... # type: Any
composite = ... # type: Any

def backref(name, **kwargs): ...
def deferred(*columns, **kw): ...

synonym = ... # type: Any
comparable_property = ... # type: Any

def compile_mappers(): ...
def clear_mappers(): ...

joinedload = ... # type: Any
joinedload_all = ... # type: Any
contains_eager = ... # type: Any
defer = ... # type: Any
undefer = ... # type: Any
undefer_group = ... # type: Any
load_only = ... # type: Any
lazyload = ... # type: Any
lazyload_all = ... # type: Any
subqueryload = ... # type: Any
subqueryload_all = ... # type: Any
immediateload = ... # type: Any
noload = ... # type: Any
defaultload = ... # type: Any

## Load = strategy_options.Load

def eagerload(*args, **kwargs): ...
def eagerload_all(*args, **kwargs): ...

contains_alias = ... # type: Any



from typing import List

# TODO group database entry object type

class struct_group:
    gr_name = ...  # type: str
    gr_passwd = ...  # type: str
    gr_gid = 0
    gr_mem = ...  # type: List[str]

def getgrgid(gid: int) -> struct_group: ...
def getgrnam(name: str) -> struct_group: ...
def getgrall() -> List[struct_group]: ...



# Stubs for concurrent.futures._base (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from collections import namedtuple

FIRST_COMPLETED = ... # type: Any
FIRST_EXCEPTION = ... # type: Any
ALL_COMPLETED = ... # type: Any
PENDING = ... # type: Any
RUNNING = ... # type: Any
CANCELLED = ... # type: Any
CANCELLED_AND_NOTIFIED = ... # type: Any
FINISHED = ... # type: Any
LOGGER = ... # type: Any

class Error(Exception): ...
class CancelledError(Error): ...
class TimeoutError(Error): ...

class _Waiter:
    event = ... # type: Any
    finished_futures = ... # type: Any
    def __init__(self): ...
    def add_result(self, future): ...
    def add_exception(self, future): ...
    def add_cancelled(self, future): ...

class _AsCompletedWaiter(_Waiter):
    lock = ... # type: Any
    def __init__(self): ...
    def add_result(self, future): ...
    def add_exception(self, future): ...
    def add_cancelled(self, future): ...

class _FirstCompletedWaiter(_Waiter):
    def add_result(self, future): ...
    def add_exception(self, future): ...
    def add_cancelled(self, future): ...

class _AllCompletedWaiter(_Waiter):
    num_pending_calls = ... # type: Any
    stop_on_exception = ... # type: Any
    lock = ... # type: Any
    def __init__(self, num_pending_calls, stop_on_exception): ...
    def add_result(self, future): ...
    def add_exception(self, future): ...
    def add_cancelled(self, future): ...

class _AcquireFutures:
    futures = ... # type: Any
    def __init__(self, futures): ...
    def __enter__(self): ...
    def __exit__(self, *args): ...

def as_completed(fs, timeout=None): ...

DoneAndNotDoneFutures = namedtuple('DoneAndNotDoneFutures', 'done not_done')

def wait(fs, timeout=None, return_when=...): ...

class Future:
    def __init__(self): ...
    def cancel(self): ...
    def cancelled(self): ...
    def running(self): ...
    def done(self): ...
    def add_done_callback(self, fn): ...
    def result(self, timeout=None): ...
    def exception(self, timeout=None): ...
    def set_running_or_notify_cancel(self): ...
    def set_result(self, result): ...
    def set_exception(self, exception): ...

class Executor:
    def submit(self, fn, *args, **kwargs): ...
    def map(self, fn, *iterables, *, timeout=None, chunksize=1): ...
    def shutdown(self, wait=True): ...
    def __enter__(self): ...
    def __exit__(self, exc_type, exc_val, exc_tb): ...



# Stubs for string

# Based on http://docs.python.org/3.2/library/string.html

from typing import Mapping

ascii_letters = ...  # type: str
ascii_lowercase = ...  # type: str
ascii_uppercase = ...  # type: str
digits = ...  # type: str
hexdigits = ...  # type: str
octdigits = ...  # type: str
punctuation = ...  # type: str
printable = ...  # type: str
whitespace = ...  # type: str

def capwords(s: str, sep: str = ...) -> str: ...

class Template:
    template = ...  # type: str

    def __init__(self, template: str) -> None: ...
    def substitute(self, mapping: Mapping[str, str], **kwds: str) -> str: ...
    def safe_substitute(self, mapping: Mapping[str, str],
                        **kwds: str) -> str: ...

# TODO Formatter



from typing import Dict

ENDMARKER = 0
NAME = 0
NUMBER = 0
STRING = 0
NEWLINE = 0
INDENT = 0
DEDENT = 0
LPAR = 0
RPAR = 0
LSQB = 0
RSQB = 0
COLON = 0
COMMA = 0
SEMI = 0
PLUS = 0
MINUS = 0
STAR = 0
SLASH = 0
VBAR = 0
AMPER = 0
LESS = 0
GREATER = 0
EQUAL = 0
DOT = 0
PERCENT = 0
LBRACE = 0
RBRACE = 0
EQEQUAL = 0
NOTEQUAL = 0
LESSEQUAL = 0
GREATEREQUAL = 0
TILDE = 0
CIRCUMFLEX = 0
LEFTSHIFT = 0
RIGHTSHIFT = 0
DOUBLESTAR = 0
PLUSEQUAL = 0
MINEQUAL = 0
STAREQUAL = 0
SLASHEQUAL = 0
PERCENTEQUAL = 0
AMPEREQUAL = 0
VBAREQUAL = 0
CIRCUMFLEXEQUAL = 0
LEFTSHIFTEQUAL = 0
RIGHTSHIFTEQUAL = 0
DOUBLESTAREQUAL = 0
DOUBLESLASH = 0
DOUBLESLASHEQUAL = 0
AT = 0
RARROW = 0
ELLIPSIS = 0
OP = 0
ERRORTOKEN = 0
N_TOKENS = 0
NT_OFFSET = 0
tok_name = {} # type: Dict[int, str]

def ISTERMINAL(x: int) -> bool: pass
def ISNONTERMINAL(x: int) -> bool: pass
def ISEOF(x: int) -> bool: pass



from typing import Iterable, Tuple, Callable, Any, AnyStr

ClientConnectedCallback = Callable[[Tuple[StreamReader, StreamWriter]], None]
import socket

from . import coroutines
from . import events
from . import protocols
from . import transports

__all__ = ... # type: str

class IncompleteReadError(EOFError):
    def __init__(self, partial: str, expected: int) -> None: ...

class LimitOverrunError(Exception):
    def __init__(self, message: str, consumed: int) -> None: ...

@coroutines.coroutine
def open_connection(
        host: str = ...,
        port: int = ...,
        *,
        loop: events.AbstractEventLoop = ...,
        limit: int = ...,
        **kwds: Any) -> Tuple[StreamReader, StreamWriter]: ...

@coroutines.coroutine
def start_server(
        client_connected_cb: ClientConnectedCallback,
        host: str = ...,
        port: int = ...,
        *,
        loop: events.AbstractEventLoop = ...,
        limit: int = ...,
        **kwds: Any) -> events.AbstractServer: ...

if hasattr(socket, 'AF_UNIX'):
    @coroutines.coroutine
    def open_unix_connection(
            path: str = ...,
            *,
            loop: events.AbstractEventLoop = ...,
            limit: int = ...,
            **kwds: Any): ...

    @coroutines.coroutine
    def start_unix_server(
        client_connected_cb: ClientConnectedCallback,
        path: str = ...,
        *,
        loop: int = ...,
        limit: int = ...,
        **kwds: Any) -> events.AbstractServer: ...

class FlowControlMixin(protocols.Protocol): ...

class StreamReaderProtocol(FlowControlMixin, protocols.Protocol):
    def __init__(self,
            stream_reader: StreamReader,
            client_connected_cb: ClientConnectedCallback = ...,
            loop: events.AbstractEventLoop = ...) -> None: ...
    def connection_made(self, transport: transports.BaseTransport) -> None: ...
    def connection_lost(self, exc: Exception) -> None: ...
    def data_received(self, data: AnyStr) -> None: ...
    def eof_received(self) -> bool: ...

class StreamWriter:
    def __init__(self,
            transport: transports.BaseTransport,
            protocol: protocols.BaseProtocol,
            reader: StreamReader,
            loop: events.AbstractEventLoop) -> None: ...
    @property
    def transport(self) -> transports.BaseTransport: ...
    def write(self, data: AnyStr) -> None: ...
    def writelines(self, data: Iterable[str]) -> None: ...
    def write_eof(self) -> None: ...
    def can_write_eof(self) -> bool: ...
    def close(self) -> None: ...
    def get_extra_info(self, name: Any, default: Any = ...) -> Any: ...
    def drain(self) -> None: ...

class StreamReader:
    def __init__(self,
            limit: int = ...,
            loop: events.AbstractEventLoop = ...) -> None: ...
    def exception(self) -> Exception: ...
    def set_exception(self, exc: Exception) -> None: ...
    def set_transport(self, transport: transports.BaseTransport) -> None: ...
    def feed_eof(self) -> None: ...
    def at_eof(self) -> bool: ...
    def feed_data(self, data: AnyStr): ...
    @coroutines.coroutine
    def readline(self) -> str: ...
    @coroutines.coroutine
    def readuntil(self, separator=b'\n') -> str: ...
    @coroutines.coroutine
    def read(self, n=-1) -> str: ...
    @coroutines.coroutine
    def readexactly(self, n) -> str: ...



from typing import TypeVar, Generic, Any, Iterable, Iterator, Callable, Tuple

_T = TypeVar('_T')

class Future(Generic[_T]):
    def cancel(self) -> bool: ...
    def cancelled(self) -> bool: ...
    def running(self) -> bool: ...
    def done(self) -> bool: ...
    def result(self, timeout: float = ...) -> _T: ...
    def exception(self, timeout: float = ...) -> Exception: ...
    def add_done_callback(self, fn: Callable[[Future], Any]) -> None: ...

    def set_running_or_notify_cancel(self) -> None: ...
    def set_result(self, result: _T) -> None: ...
    def set_exception(self, exception: Exception) -> None: ...

class Executor:
    def submit(self, fn: Callable[..., _T], *args: Any, **kwargs: Any) -> Future[_T]: ...
    def map(self, func: Callable[..., _T], *iterables: Any, timeout: float = ...) -> Iterable[_T]: ...
    def shutdown(self, wait: bool = ...) -> None: ...
    def __enter__(self) -> Executor: ...
    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> bool: ...

class ThreadPoolExecutor(Executor):
    def __init__(self, max_workers: int) -> None: ...

class ProcessPoolExecutor(Executor):
    def __init__(self, max_workers: None) -> None: ...

def wait(fs: Iterable[Future], timeout: float = ..., return_when: str = ...) -> Tuple[Iterable[Future], Iterable[Future]]: ...

FIRST_COMPLETED = ...  # type: str
FIRST_EXCEPTION = ...  # type: str
ALL_COMPLETED = ...  # type: str

def as_completed(fs: Iterable[Future], timeout: float = ...) -> Iterator[Future]: ...



"""Parallel subprocess task runner.

This is used for running mypy tests.
"""

from typing import Dict, List, Optional, Set, Tuple

import os
import pipes
import re
from subprocess import Popen, STDOUT
import sys
import tempfile
import time


class WaiterError(Exception):
    pass


class LazySubprocess:
    """Wrapper around a subprocess that runs a test task."""

    def __init__(self, name: str, args: List[str], *, cwd: str = None,
                 env: Dict[str, str] = None) -> None:
        self.name = name
        self.args = args
        self.cwd = cwd
        self.env = env
        self.start_time = None  # type: float
        self.end_time = None  # type: float

    def start(self) -> None:
        self.outfile = tempfile.NamedTemporaryFile()
        self.start_time = time.time()
        self.process = Popen(self.args, cwd=self.cwd, env=self.env,
                             stdout=self.outfile, stderr=STDOUT)
        self.pid = self.process.pid

    def handle_exit_status(self, status: int) -> None:
        """Update process exit status received via an external os.waitpid() call."""
        # Inlined subprocess._handle_exitstatus, it's not a public API.
        # TODO(jukka): I'm not quite sure why this is implemented like this.
        self.end_time = time.time()
        process = self.process
        assert process.returncode is None
        if os.WIFSIGNALED(status):
            process.returncode = -os.WTERMSIG(status)
        elif os.WIFEXITED(status):
            process.returncode = os.WEXITSTATUS(status)
        else:
            # Should never happen
            raise RuntimeError("Unknown child exit status!")
        assert process.returncode is not None

    def wait(self) -> int:
        return self.process.wait()

    def status(self) -> Optional[int]:
        return self.process.returncode

    def read_output(self) -> str:
        with open(self.outfile.name, 'rb') as file:
            # Assume it's ascii to avoid unicode headaches (and portability issues).
            return file.read().decode('ascii')

    def cleanup(self) -> None:
        self.outfile.close()
        assert not os.path.exists(self.outfile.name)

    @property
    def elapsed_time(self) -> float:
        return self.end_time - self.start_time


class Noter:
    """Update stats about running jobs.

    Only used when verbosity == 0.
    """

    def __init__(self, total: int) -> None:
        # Total number of tasks.
        self.total = total
        self.running = set()  # type: Set[int]
        # Passed tasks.
        self.passes = 0
        # Failed tasks.
        self.fails = 0

    def start(self, job: int) -> None:
        self.running.add(job)
        self.update()

    def stop(self, job: int, failed: bool) -> None:
        self.running.remove(job)
        if failed:
            self.fails += 1
        else:
            self.passes += 1
        self.update()

    def message(self, msg: str) -> None:
        # Using a CR instead of NL will overwrite the line.
        sys.stdout.write('%-80s\r' % msg)
        sys.stdout.flush()

    def update(self) -> None:
        pending = self.total - self.passes - self.fails - len(self.running)
        args = (self.passes, self.fails, pending, len(self.running))
        msg = 'passed %d, failed %d, pending %d; running %d' % args
        self.message(msg)

    def clear(self) -> None:
        self.message('')


class Waiter:
    """Run subprocesses in parallel and wait for them.

    Usage:

    waiter = Waiter()
    waiter.add('sleep 9')
    waiter.add('sleep 10')
    if not waiter.run():
        print('error')
    """
    def __init__(self, limit: int = 0, *, verbosity: int = 0, xfail: List[str] = []) -> None:
        self.verbosity = verbosity
        self.queue = []  # type: List[LazySubprocess]
        # Index of next task to run in the queue.
        self.next = 0
        self.current = {}  # type: Dict[int, Tuple[int, LazySubprocess]]
        if limit == 0:
            try:
                sched_getaffinity = os.sched_getaffinity
            except AttributeError:
                limit = 2
            else:
                # Note: only count CPUs we are allowed to use. It is a
                # major mistake to count *all* CPUs on the machine.
                limit = len(sched_getaffinity(0))
        self.limit = limit
        assert limit > 0
        self.xfail = set(xfail)
        self._note = None  # type: Noter
        self.times1 = {}  # type: Dict[str, float]
        self.times2 = {}  # type: Dict[str, float]

    def add(self, cmd: LazySubprocess) -> int:
        rv = len(self.queue)
        self.queue.append(cmd)
        return rv

    def _start_next(self) -> None:
        num = self.next
        cmd = self.queue[num]
        name = cmd.name
        cmd.start()
        self.current[cmd.pid] = (num, cmd)
        if self.verbosity >= 1:
            print('%-8s #%d %s' % ('START', num, name))
            if self.verbosity >= 2:
                print('%-8s #%d %s' % ('CWD', num, cmd.cwd or '.'))
                cmd_str = ' '.join(pipes.quote(a) for a in cmd.args)
                print('%-8s #%d %s' % ('COMMAND', num, cmd_str))
            sys.stdout.flush()
        elif self.verbosity >= 0:
            self._note.start(num)
        self.next += 1

    def _record_time(self, name: str, elapsed_time: float) -> None:
        # The names we use are space-separated series of rather arbitrary words.
        # They tend to start general and get more specific, so use that.
        name1 = re.sub(' .*', '', name)  # First word.
        self.times1[name1] = elapsed_time + self.times1.get(name1, 0)
        name2 = re.sub('( .*?) .*', r'\1', name)  # First two words.
        self.times2[name2] = elapsed_time + self.times2.get(name2, 0)

    def _wait_next(self) -> Tuple[List[str], int, int]:
        """Wait for a single task to finish.

        Return tuple (list of failed tasks, number test cases, number of failed tests).
        """
        pid, status = os.waitpid(-1, 0)
        num, cmd = self.current.pop(pid)
        name = cmd.name

        cmd.handle_exit_status(status)

        self._record_time(cmd.name, cmd.elapsed_time)

        rc = cmd.wait()
        if rc >= 0:
            msg = 'EXIT %d' % rc
        else:
            msg = 'SIG %d' % -rc
        if self.verbosity >= 1:
            print('%-8s #%d %s' % (msg, num, name))
            sys.stdout.flush()
        elif self.verbosity >= 0:
            self._note.stop(num, bool(rc))
        elif self.verbosity >= -1:
            sys.stdout.write('.' if rc == 0 else msg[0])
            num_complete = self.next - len(self.current)
            if num_complete % 50 == 0 or num_complete == len(self.queue):
                sys.stdout.write(' %d/%d\n' % (num_complete, len(self.queue)))
            elif num_complete % 10 == 0:
                sys.stdout.write(' ')
            sys.stdout.flush()

        if rc != 0:
            if name not in self.xfail:
                fail_type = 'FAILURE'
            else:
                fail_type = 'XFAIL'
        else:
            if name not in self.xfail:
                fail_type = None
            else:
                fail_type = 'UPASS'

        # Get task output.
        output = cmd.read_output()
        cmd.cleanup()
        num_tests, num_tests_failed = parse_test_stats_from_output(output, fail_type)

        if fail_type is not None or self.verbosity >= 1:
            self._report_task_failure(fail_type, num, name, output)

        if fail_type is not None:
            failed_tasks = ['%8s %s' % (fail_type, name)]
        else:
            failed_tasks = []

        return failed_tasks, num_tests, num_tests_failed

    def _report_task_failure(self, fail_type: Optional[str], num: int, name: str,
                             output: str) -> None:
        if self.verbosity <= 0:
            sys.stdout.write('\n')
        sys.stdout.write('\n%-8s #%d %s\n\n' % (fail_type or 'PASS', num, name))
        sys.stdout.write(output + '\n')
        sys.stdout.flush()

    def run(self) -> int:
        if self.verbosity >= -1:
            print('%-8s %d' % ('PARALLEL', self.limit))
            sys.stdout.flush()
        if self.verbosity == 0:
            self._note = Noter(len(self.queue))
        print('SUMMARY  %d tasks selected' % len(self.queue))
        sys.stdout.flush()
        # Failed tasks.
        all_failures = []  # type: List[str]
        # Number of test cases. Some tasks can involve multiple test cases.
        total_tests = 0
        # Number of failed test cases.
        total_failed_tests = 0
        while self.current or self.next < len(self.queue):
            while len(self.current) < self.limit and self.next < len(self.queue):
                self._start_next()
            fails, tests, test_fails = self._wait_next()
            all_failures += fails
            total_tests += tests
            total_failed_tests += test_fails
        if self.verbosity == 0:
            self._note.clear()
        if all_failures:
            summary = 'SUMMARY  %d/%d tasks and %d/%d tests failed' % (
                len(all_failures), len(self.queue), total_failed_tests, total_tests)
            print(summary)
            for f in all_failures:
                print(f)
            print(summary)
            print('*** FAILURE ***')
            sys.stdout.flush()
            if any('XFAIL' not in f for f in all_failures):
                return 1
        else:
            print('SUMMARY  all %d tasks and %d tests passed' % (
                len(self.queue), total_tests))
            print('*** OK ***')
            sys.stdout.flush()

        return 0


def parse_test_stats_from_output(output: str, fail_type: Optional[str]) -> Tuple[int, int]:
    """Parse tasks output and determine test counts.

    Return tuple (number of tests, number of test failures). Default
    to the entire task representing a single test as a fallback.
    """
    m = re.search('^([0-9]+)/([0-9]+) test cases failed(, ([0-9]+) skipped)?.$', output,
                  re.MULTILINE)
    if m:
        return int(m.group(2)), int(m.group(1))
    m = re.search('^([0-9]+) test cases run(, ([0-9]+) skipped)?, all passed.$', output,
                  re.MULTILINE)
    if m:
        return int(m.group(1)), 0
    # Couldn't find test counts, so fall back to single test per tasks.
    if fail_type is not None:
        return 1, 1
    else:
        return 1, 0



# Stubs for _curses (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

ALL_MOUSE_EVENTS = ... # type: int
A_ALTCHARSET = ... # type: int
A_ATTRIBUTES = ... # type: int
A_BLINK = ... # type: int
A_BOLD = ... # type: int
A_CHARTEXT = ... # type: int
A_COLOR = ... # type: int
A_DIM = ... # type: int
A_HORIZONTAL = ... # type: int
A_INVIS = ... # type: int
A_LEFT = ... # type: int
A_LOW = ... # type: int
A_NORMAL = ... # type: int
A_PROTECT = ... # type: int
A_REVERSE = ... # type: int
A_RIGHT = ... # type: int
A_STANDOUT = ... # type: int
A_TOP = ... # type: int
A_UNDERLINE = ... # type: int
A_VERTICAL = ... # type: int
BUTTON1_CLICKED = ... # type: int
BUTTON1_DOUBLE_CLICKED = ... # type: int
BUTTON1_PRESSED = ... # type: int
BUTTON1_RELEASED = ... # type: int
BUTTON1_TRIPLE_CLICKED = ... # type: int
BUTTON2_CLICKED = ... # type: int
BUTTON2_DOUBLE_CLICKED = ... # type: int
BUTTON2_PRESSED = ... # type: int
BUTTON2_RELEASED = ... # type: int
BUTTON2_TRIPLE_CLICKED = ... # type: int
BUTTON3_CLICKED = ... # type: int
BUTTON3_DOUBLE_CLICKED = ... # type: int
BUTTON3_PRESSED = ... # type: int
BUTTON3_RELEASED = ... # type: int
BUTTON3_TRIPLE_CLICKED = ... # type: int
BUTTON4_CLICKED = ... # type: int
BUTTON4_DOUBLE_CLICKED = ... # type: int
BUTTON4_PRESSED = ... # type: int
BUTTON4_RELEASED = ... # type: int
BUTTON4_TRIPLE_CLICKED = ... # type: int
BUTTON_ALT = ... # type: int
BUTTON_CTRL = ... # type: int
BUTTON_SHIFT = ... # type: int
COLOR_BLACK = ... # type: int
COLOR_BLUE = ... # type: int
COLOR_CYAN = ... # type: int
COLOR_GREEN = ... # type: int
COLOR_MAGENTA = ... # type: int
COLOR_RED = ... # type: int
COLOR_WHITE = ... # type: int
COLOR_YELLOW = ... # type: int
ERR = ... # type: int
KEY_A1 = ... # type: int
KEY_A3 = ... # type: int
KEY_B2 = ... # type: int
KEY_BACKSPACE = ... # type: int
KEY_BEG = ... # type: int
KEY_BREAK = ... # type: int
KEY_BTAB = ... # type: int
KEY_C1 = ... # type: int
KEY_C3 = ... # type: int
KEY_CANCEL = ... # type: int
KEY_CATAB = ... # type: int
KEY_CLEAR = ... # type: int
KEY_CLOSE = ... # type: int
KEY_COMMAND = ... # type: int
KEY_COPY = ... # type: int
KEY_CREATE = ... # type: int
KEY_CTAB = ... # type: int
KEY_DC = ... # type: int
KEY_DL = ... # type: int
KEY_DOWN = ... # type: int
KEY_EIC = ... # type: int
KEY_END = ... # type: int
KEY_ENTER = ... # type: int
KEY_EOL = ... # type: int
KEY_EOS = ... # type: int
KEY_EXIT = ... # type: int
KEY_F0 = ... # type: int
KEY_F1 = ... # type: int
KEY_F10 = ... # type: int
KEY_F11 = ... # type: int
KEY_F12 = ... # type: int
KEY_F13 = ... # type: int
KEY_F14 = ... # type: int
KEY_F15 = ... # type: int
KEY_F16 = ... # type: int
KEY_F17 = ... # type: int
KEY_F18 = ... # type: int
KEY_F19 = ... # type: int
KEY_F2 = ... # type: int
KEY_F20 = ... # type: int
KEY_F21 = ... # type: int
KEY_F22 = ... # type: int
KEY_F23 = ... # type: int
KEY_F24 = ... # type: int
KEY_F25 = ... # type: int
KEY_F26 = ... # type: int
KEY_F27 = ... # type: int
KEY_F28 = ... # type: int
KEY_F29 = ... # type: int
KEY_F3 = ... # type: int
KEY_F30 = ... # type: int
KEY_F31 = ... # type: int
KEY_F32 = ... # type: int
KEY_F33 = ... # type: int
KEY_F34 = ... # type: int
KEY_F35 = ... # type: int
KEY_F36 = ... # type: int
KEY_F37 = ... # type: int
KEY_F38 = ... # type: int
KEY_F39 = ... # type: int
KEY_F4 = ... # type: int
KEY_F40 = ... # type: int
KEY_F41 = ... # type: int
KEY_F42 = ... # type: int
KEY_F43 = ... # type: int
KEY_F44 = ... # type: int
KEY_F45 = ... # type: int
KEY_F46 = ... # type: int
KEY_F47 = ... # type: int
KEY_F48 = ... # type: int
KEY_F49 = ... # type: int
KEY_F5 = ... # type: int
KEY_F50 = ... # type: int
KEY_F51 = ... # type: int
KEY_F52 = ... # type: int
KEY_F53 = ... # type: int
KEY_F54 = ... # type: int
KEY_F55 = ... # type: int
KEY_F56 = ... # type: int
KEY_F57 = ... # type: int
KEY_F58 = ... # type: int
KEY_F59 = ... # type: int
KEY_F6 = ... # type: int
KEY_F60 = ... # type: int
KEY_F61 = ... # type: int
KEY_F62 = ... # type: int
KEY_F63 = ... # type: int
KEY_F7 = ... # type: int
KEY_F8 = ... # type: int
KEY_F9 = ... # type: int
KEY_FIND = ... # type: int
KEY_HELP = ... # type: int
KEY_HOME = ... # type: int
KEY_IC = ... # type: int
KEY_IL = ... # type: int
KEY_LEFT = ... # type: int
KEY_LL = ... # type: int
KEY_MARK = ... # type: int
KEY_MAX = ... # type: int
KEY_MESSAGE = ... # type: int
KEY_MIN = ... # type: int
KEY_MOUSE = ... # type: int
KEY_MOVE = ... # type: int
KEY_NEXT = ... # type: int
KEY_NPAGE = ... # type: int
KEY_OPEN = ... # type: int
KEY_OPTIONS = ... # type: int
KEY_PPAGE = ... # type: int
KEY_PREVIOUS = ... # type: int
KEY_PRINT = ... # type: int
KEY_REDO = ... # type: int
KEY_REFERENCE = ... # type: int
KEY_REFRESH = ... # type: int
KEY_REPLACE = ... # type: int
KEY_RESET = ... # type: int
KEY_RESIZE = ... # type: int
KEY_RESTART = ... # type: int
KEY_RESUME = ... # type: int
KEY_RIGHT = ... # type: int
KEY_SAVE = ... # type: int
KEY_SBEG = ... # type: int
KEY_SCANCEL = ... # type: int
KEY_SCOMMAND = ... # type: int
KEY_SCOPY = ... # type: int
KEY_SCREATE = ... # type: int
KEY_SDC = ... # type: int
KEY_SDL = ... # type: int
KEY_SELECT = ... # type: int
KEY_SEND = ... # type: int
KEY_SEOL = ... # type: int
KEY_SEXIT = ... # type: int
KEY_SF = ... # type: int
KEY_SFIND = ... # type: int
KEY_SHELP = ... # type: int
KEY_SHOME = ... # type: int
KEY_SIC = ... # type: int
KEY_SLEFT = ... # type: int
KEY_SMESSAGE = ... # type: int
KEY_SMOVE = ... # type: int
KEY_SNEXT = ... # type: int
KEY_SOPTIONS = ... # type: int
KEY_SPREVIOUS = ... # type: int
KEY_SPRINT = ... # type: int
KEY_SR = ... # type: int
KEY_SREDO = ... # type: int
KEY_SREPLACE = ... # type: int
KEY_SRESET = ... # type: int
KEY_SRIGHT = ... # type: int
KEY_SRSUME = ... # type: int
KEY_SSAVE = ... # type: int
KEY_SSUSPEND = ... # type: int
KEY_STAB = ... # type: int
KEY_SUNDO = ... # type: int
KEY_SUSPEND = ... # type: int
KEY_UNDO = ... # type: int
KEY_UP = ... # type: int
OK = ... # type: int
REPORT_MOUSE_POSITION = ... # type: int
_C_API = ... # type: Any
version = ... # type: bytes

def baudrate(*args, **kwargs): ...
def beep(*args, **kwargs): ...
def can_change_color(*args, **kwargs): ...
def cbreak(*args, **kwargs): ...
def color_content(*args, **kwargs): ...
def color_pair(*args, **kwargs): ...
def curs_set(*args, **kwargs): ...
def def_prog_mode(*args, **kwargs): ...
def def_shell_mode(*args, **kwargs): ...
def delay_output(*args, **kwargs): ...
def doupdate(*args, **kwargs): ...
def echo(*args, **kwargs): ...
def endwin(*args, **kwargs): ...
def erasechar(*args, **kwargs): ...
def filter(*args, **kwargs): ...
def flash(*args, **kwargs): ...
def flushinp(*args, **kwargs): ...
def getmouse(*args, **kwargs): ...
def getsyx(*args, **kwargs): ...
def getwin(*args, **kwargs): ...
def halfdelay(*args, **kwargs): ...
def has_colors(*args, **kwargs): ...
def has_ic(*args, **kwargs): ...
def has_il(*args, **kwargs): ...
def has_key(*args, **kwargs): ...
def init_color(*args, **kwargs): ...
def init_pair(*args, **kwargs): ...
def initscr(*args, **kwargs): ...
def intrflush(*args, **kwargs): ...
def is_term_resized(*args, **kwargs): ...
def isendwin(*args, **kwargs): ...
def keyname(*args, **kwargs): ...
def killchar(*args, **kwargs): ...
def longname(*args, **kwargs): ...
def meta(*args, **kwargs): ...
def mouseinterval(*args, **kwargs): ...
def mousemask(*args, **kwargs): ...
def napms(*args, **kwargs): ...
def newpad(*args, **kwargs): ...
def newwin(*args, **kwargs): ...
def nl(*args, **kwargs): ...
def nocbreak(*args, **kwargs): ...
def noecho(*args, **kwargs): ...
def nonl(*args, **kwargs): ...
def noqiflush(*args, **kwargs): ...
def noraw(*args, **kwargs): ...
def pair_content(*args, **kwargs): ...
def pair_number(*args, **kwargs): ...
def putp(*args, **kwargs): ...
def qiflush(*args, **kwargs): ...
def raw(*args, **kwargs): ...
def reset_prog_mode(*args, **kwargs): ...
def reset_shell_mode(*args, **kwargs): ...
def resetty(*args, **kwargs): ...
def resize_term(*args, **kwargs): ...
def resizeterm(*args, **kwargs): ...
def savetty(*args, **kwargs): ...
def setsyx(*args, **kwargs): ...
def setupterm(*args, **kwargs): ...
def start_color(*args, **kwargs): ...
def termattrs(*args, **kwargs): ...
def termname(*args, **kwargs): ...
def tigetflag(*args, **kwargs): ...
def tigetnum(*args, **kwargs): ...
def tigetstr(*args, **kwargs): ...
def tparm(*args, **kwargs): ...
def typeahead(*args, **kwargs): ...
def unctrl(*args, **kwargs): ...
def unget_wch(*args, **kwargs): ...
def ungetch(*args, **kwargs): ...
def ungetmouse(*args, **kwargs): ...
def update_lines_cols(*args, **kwargs): ...
def use_default_colors(*args, **kwargs): ...
def use_env(*args, **kwargs): ...

class error(Exception): ...



# Stubs for email.mime.text (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from email.mime.nonmultipart import MIMENonMultipart

class MIMEText(MIMENonMultipart):
    def __init__(self, _text, _subtype=..., _charset=...) -> None: ...



# Stubs for re
# Ron Murawski <ron@horizonchess.com>
# 'bytes' support added by Jukka Lehtosalo

# based on: http://docs.python.org/3.2/library/re.html
# and http://hg.python.org/cpython/file/618ea5612e83/Lib/re.py

from typing import (
    List, Iterator, Callable, Tuple, Sequence, Dict, Union,
    Generic, AnyStr, Match, Pattern
)

# ----- re variables and constants -----
A = 0
ASCII = 0
DEBUG = 0
I = 0
IGNORECASE = 0
L = 0
LOCALE = 0
M = 0
MULTILINE = 0
S = 0
DOTALL = 0
X = 0
VERBOSE = 0
U = 0
UNICODE = 0

class error(Exception): ...

def compile(pattern: AnyStr, flags: int = ...) -> Pattern[AnyStr]: ...
def search(pattern: AnyStr, string: AnyStr,
           flags: int = ...) -> Match[AnyStr]: ...
def match(pattern: AnyStr, string: AnyStr,
          flags: int = ...) -> Match[AnyStr]: ...
def split(pattern: AnyStr, string: AnyStr, maxsplit: int = ...,
          flags: int = ...) -> List[AnyStr]: ...
def findall(pattern: AnyStr, string: AnyStr,
            flags: int = ...) -> List[AnyStr]: ...

# Return an iterator yielding match objects over all non-overlapping matches
# for the RE pattern in string. The string is scanned left-to-right, and
# matches are returned in the order found. Empty matches are included in the
# result unless they touch the beginning of another match.
def finditer(pattern: AnyStr, string: AnyStr,
             flags: int = ...) -> Iterator[Match[AnyStr]]: ...

def sub(pattern: AnyStr, repl: Union[AnyStr, Callable[[Match[AnyStr]], AnyStr]],
        string: AnyStr, count: int = ..., flags: int = ...) -> AnyStr: ...

def subn(pattern: AnyStr, repl: Union[AnyStr, Callable[[Match[AnyStr]], AnyStr]],
         string: AnyStr, count: int = ..., flags: int = ...) -> Tuple[AnyStr, int]:
    ...

def escape(string: AnyStr) -> AnyStr: ...

def purge() -> None: ...



# Stubs for textwrap (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class _unicode: ...

class TextWrapper:
    whitespace_trans = ... # type: Any
    unicode_whitespace_trans = ... # type: Any
    uspace = ... # type: Any
    wordsep_re = ... # type: Any
    wordsep_simple_re = ... # type: Any
    sentence_end_re = ... # type: Any
    width = ... # type: Any
    initial_indent = ... # type: Any
    subsequent_indent = ... # type: Any
    expand_tabs = ... # type: Any
    replace_whitespace = ... # type: Any
    fix_sentence_endings = ... # type: Any
    break_long_words = ... # type: Any
    drop_whitespace = ... # type: Any
    break_on_hyphens = ... # type: Any
    wordsep_re_uni = ... # type: Any
    wordsep_simple_re_uni = ... # type: Any
    def __init__(self, width=..., initial_indent=..., subsequent_indent=..., expand_tabs=..., replace_whitespace=..., fix_sentence_endings=..., break_long_words=..., drop_whitespace=..., break_on_hyphens=...) -> None: ...
    def wrap(self, text): ...
    def fill(self, text): ...

def wrap(text, width=..., **kwargs): ...
def fill(text, width=..., **kwargs): ...
def dedent(text): ...



# Stubs for email.contentmanager (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class ContentManager:
    get_handlers = ...  # type: Any
    set_handlers = ...  # type: Any
    def __init__(self) -> None: ...
    def add_get_handler(self, key, handler): ...
    def get_content(self, msg, *args, **kw): ...
    def add_set_handler(self, typekey, handler): ...
    def set_content(self, msg, obj, *args, **kw): ...

raw_data_manager = ...  # type: Any

def get_text_content(msg, errors=...): ...
def get_non_text_content(msg): ...
def get_message_content(msg): ...
def get_and_fixup_unknown_message_content(msg): ...
def set_text_content(msg, string, subtype=..., charset=..., cte=..., disposition=...,
                     filename=..., cid=..., params=..., headers=...): ...
def set_message_content(msg, message, subtype=..., cte=..., disposition=..., filename=...,
                        cid=..., params=..., headers=...): ...
def set_bytes_content(msg, data, maintype, subtype, cte=..., disposition=..., filename=...,
                      cid=..., params=..., headers=...): ...



# Generated by pytype, with only minor tweaks. Might be incomplete.

from typing import Any, Optional, List, Callable, Tuple, Dict, Iterable, Union

# See https://groups.google.com/forum/#!topic/python-ideas/gA1gdj3RZ5g
Text = Union[str, unicode]

NO_DEFAULT = ...  # type: Tuple[str, ...]
SUPPRESS_HELP = ...  # type: str
SUPPRESS_USAGE = ...  # type: str

def check_builtin(option: Option, opt, value: Text) -> Any: ...
def check_choice(option: Option, opt, value) -> Any: ...
def isbasestring(x) -> bool: ...

class OptParseError(Exception):
    msg = ...  # type: Any
    def __init__(self, msg) -> None: ...

class BadOptionError(OptParseError):
    __doc__ = ...  # type: str
    opt_str = ...  # type: Any
    def __init__(self, opt_str) -> None: ...

class AmbiguousOptionError(BadOptionError):
    possibilities = ...  # type: Any
    def __init__(self, opt_str, possibilities) -> None: ...

class OptionError(OptParseError):
    msg = ...  # type: Any
    option_id = ...  # type: str
    def __init__(self, msg, option: Option) -> None: ...

class OptionConflictError(OptionError): ...

class HelpFormatter:
    NO_DEFAULT_VALUE = ...  # type: str
    _long_opt_fmt = ...  # type: Union[str, unicode]
    _short_opt_fmt = ...  # type: Union[str, unicode]
    current_indent = ...  # type: int
    default_tag = ...  # type: str
    help_position = ...  # type: Any
    help_width = ...  # type: Any
    indent_increment = ...  # type: Any
    level = ...  # type: int
    max_help_position = ...  # type: int
    option_strings = ...  # type: Dict[Option, str]
    parser = ...  # type: Any
    short_first = ...  # type: Any
    width = ...  # type: Any
    def __init__(self, indent_increment, max_help_position, width, short_first) -> None: ...
    def _format_text(self, text: Text) -> Text: ...
    def dedent(self) -> None: ...
    def expand_default(self, option: Option) -> Text: ...
    def format_description(self, description) -> Any: ...
    def format_epilog(self, epilog) -> Any: ...
    def format_heading(self, heading) -> Any: ...
    def format_option(self, option: Any) -> str: ...
    def format_option_strings(self, option: Any) -> Any: ...
    def format_usage(self, usage) -> Any: ...
    def indent(self) -> None: ...
    def set_long_opt_delimiter(self, delim) -> None: ...
    def set_parser(self, parser) -> None: ...
    def set_short_opt_delimiter(self, delim) -> None: ...
    def store_option_strings(self, parser) -> None: ...

class IndentedHelpFormatter(HelpFormatter):
    __doc__ = ...  # type: str
    _long_opt_fmt = ...  # type: str
    _short_opt_fmt = ...  # type: str
    current_indent = ...  # type: int
    default_tag = ...  # type: str
    help_position = ...  # type: int
    help_width = ...  # type: Optional[int]
    indent_increment = ...  # type: Any
    level = ...  # type: int
    max_help_position = ...  # type: int
    option_strings = ...  # type: Dict[Any, Any]
    parser = ...  # type: Optional[OptionParser]
    short_first = ...  # type: Any
    width = ...  # type: Any
    def __init__(self, *args, **kwargs) -> None: ...
    def format_heading(self, heading) -> str: ...
    def format_usage(self, usage) -> str: ...

class Option:
    ACTIONS = ...  # type: Tuple[str, ...]
    ALWAYS_TYPED_ACTIONS = ...  # type: Tuple[str, ...]
    ATTRS = ...  # type: List[str]
    CHECK_METHODS = ...  # type: Union[None, List[Callable]]
    CONST_ACTIONS = ...  # type: Tuple[str, ...]
    STORE_ACTIONS = ...  # type: Tuple[str, ...]
    TYPED_ACTIONS = ...  # type: Tuple[str, ...]
    TYPES = ...  # type: Tuple[str, ...]
    TYPE_CHECKER = ...  # type: Dict[str, Callable]
    __doc__ = ...  # type: str
    _long_opts = ...  # type: List[Text]
    _short_opts = ...  # type: List[Text]
    action = ...  # type: str
    dest = ...  # type: Any
    nargs = ...  # type: int
    type = ...  # type: Any
    def __init__(self, *args, **kwargs) -> None: ...
    def __repr__(self) -> str: ...
    def __str__(self) -> str: ...
    def _check_action(self) -> None: ...
    def _check_callback(self) -> None: ...
    def _check_choice(self) -> None: ...
    def _check_const(self) -> None: ...
    def _check_dest(self) -> None: ...
    def _check_nargs(self) -> None: ...
    def _check_opt_strings(self, opts) -> Any: ...
    def _check_type(self) -> None: ...
    def _set_attrs(self, attrs: Dict[str, Any]) -> None: ...
    def _set_opt_strings(self, opts) -> None: ...
    def check_value(self, opt, value) -> Any: ...
    def convert_value(self, opt, value) -> Any: ...
    def get_opt_string(self) -> Text: ...
    def process(self, opt, value, values: Any, parser: OptionParser) -> int: ...
    def take_action(self, action, dest, opt, value, values, parser: OptionParser) -> int: ...
    def takes_value(self) -> bool: ...

make_option = Option

class OptionContainer:
    _long_opt = ...  # type: Dict[Text, Any]
    _short_opt = ...  # type: Dict[Text, Any]
    conflict_handler = ...  # type: Any
    defaults = ...  # type: Dict[Text, Any]
    description = ...  # type: Any
    option_class = ...  # type: Any
    def __init__(self, option_class, conflict_handler, description) -> None: ...
    def _check_conflict(self, option: Any) -> None: ...
    def _create_option_mappings(self) -> None: ...
    def _share_option_mappings(self, parser) -> None: ...
    def add_option(self, *args, **kwargs) -> Any: ...
    def add_options(self, option_list) -> None: ...
    def destroy(self) -> None: ...
    def format_description(self, formatter: Optional[HelpFormatter]) -> Any: ...
    def format_help(self, formatter: HelpFormatter) -> str: ...
    def format_option_help(self, formatter: Optional[HelpFormatter]) -> str: ...
    def get_description(self) -> Any: ...
    def get_option(self, opt_str) -> Optional[Option]: ...
    def has_option(self, opt_str) -> bool: ...
    def remove_option(self, opt_str) -> None: ...
    def set_conflict_handler(self, handler) -> None: ...
    def set_description(self, description) -> None: ...

class OptionGroup(OptionContainer):
    _long_opt = ...  # type: Dict[Any, Any]
    _short_opt = ...  # type: Dict[Any, Any]
    conflict_handler = ...  # type: Any
    defaults = ...  # type: Dict[Text, Any]
    description = ...  # type: Any
    option_class = ...  # type: Any
    option_list = ...  # type: List
    parser = ...  # type: Any
    title = ...  # type: Any
    def __init__(self, parser, title, *args, **kwargs) -> None: ...
    def _create_option_list(self) -> None: ...
    def format_help(self, formatter: HelpFormatter) -> Any: ...
    def set_title(self, title) -> None: ...

class OptionParser(OptionContainer):
    __doc__ = ...  # type: str
    _long_opt = ...  # type: Dict[Text, Any]
    _short_opt = ...  # type: Dict[Any, Any]
    allow_interspersed_args = ...  # type: bool
    conflict_handler = ...  # type: Any
    defaults = ...  # type: Dict[Any, Any]
    description = ...  # type: Text
    epilog = ...  # type: Any
    formatter = ...  # type: HelpFormatter
    largs = ...  # type: Union[None, List[Text]]
    option_class = ...  # type: Callable
    option_groups = ...  # type: List[OptionParser]
    option_list = ...  # type: List[Any]
    process_default_values = ...  # type: Any
    prog = ...  # type: Any
    rargs = ...  # type: Optional[List[Any]]
    standard_option_list = ...  # type: List
    usage = ...  # type: Optional[Text]
    values = ...  # type: Any
    version = ...  # type: Text
    def __init__(self, *args, **kwargs) -> None: ...
    def _add_help_option(self) -> None: ...
    def _add_version_option(self) -> None: ...
    def _create_option_list(self) -> None: ...
    def _get_all_options(self) -> List[Any]: ...
    def _get_args(self, args: Iterable) -> List[Any]: ...
    def _get_encoding(self, file) -> Any: ...
    def _init_parsing_state(self) -> None: ...
    def _match_long_opt(self, opt) -> Any: ...
    def _populate_option_list(self, option_list, *args, **kwargs) -> None: ...
    def _process_args(self, largs: List[Text], rargs: List, values: Values) -> None: ...
    def _process_long_opt(self, rargs: List, values) -> None: ...
    def _process_short_opts(self, rargs: List, values) -> None: ...
    def add_option_group(self, *args, **kwargs) -> OptionParser: ...
    def check_values(self, values, args) -> Tuple[Any, ...]: ...
    def disable_interspersed_args(self) -> None: ...
    def enable_interspersed_args(self) -> None: ...
    def error(self, msg) -> None: ...
    def exit(self, *args, **kwargs) -> None: ...
    def expand_prog_name(self, s: Optional[Text]) -> Any: ...
    def format_epilog(self, formatter: Union[HelpFormatter, OptionParser, None]) -> Any: ...
    def format_help(self, *args, **kwargs) -> str: ...
    def format_option_help(self, *args, **kwargs) -> str: ...
    def get_default_values(self) -> Values: ...
    def get_option_group(self, opt_str) -> Any: ...
    def get_prog_name(self) -> Any: ...
    def get_usage(self) -> Text: ...
    def get_version(self) -> Any: ...
    def parse_args(self, *args, **kwargs) -> Tuple[Any, ...]: ...
    def print_help(self, *args, **kwargs) -> None: ...
    def print_usage(self, *args, **kwargs) -> None: ...
    def print_version(self, *args, **kwargs) -> None: ...
    def set_default(self, dest, value) -> None: ...
    def set_defaults(self, *args, **kwargs) -> None: ...
    def set_process_default_values(self, process) -> None: ...
    def set_usage(self, usage: Text) -> None: ...

class OptionValueError(OptParseError):
    __doc__ = ...  # type: str
    msg = ...  # type: Any

class TitledHelpFormatter(HelpFormatter):
    __doc__ = ...  # type: str
    _long_opt_fmt = ...  # type: str
    _short_opt_fmt = ...  # type: str
    current_indent = ...  # type: int
    default_tag = ...  # type: str
    help_position = ...  # type: int
    help_width = ...  # type: None
    indent_increment = ...  # type: Any
    level = ...  # type: int
    max_help_position = ...  # type: int
    option_strings = ...  # type: Dict
    parser = ...  # type: None
    short_first = ...  # type: Any
    width = ...  # type: Any
    def __init__(self, *args, **kwargs) -> None: ...
    def format_heading(self, heading) -> str: ...
    def format_usage(self, usage) -> str: ...

class Values:
    def __cmp__(self, other) -> int: ...
    def __init__(self, *args, **kwargs) -> None: ...
    def __repr__(self) -> str: ...
    def __str__(self) -> str: ...
    def _update(self, dict: Dict[str, Any], mode) -> None: ...
    def _update_careful(self, dict: Dict[str, Any]) -> None: ...
    def _update_loose(self, dict) -> None: ...
    def ensure_value(self, attr, value) -> Any: ...
    def read_file(self, filename, *args, **kwargs) -> None: ...
    def read_module(self, modname, *args, **kwargs) -> None: ...




# Stubs for requests.packages.urllib3.util.request (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
#from ..packages import six

#b = six.b

ACCEPT_ENCODING = ...  # type: Any

def make_headers(keep_alive=..., accept_encoding=..., user_agent=..., basic_auth=..., proxy_basic_auth=..., disable_cache=...): ...



from .elements import ClauseElement
from .base import Executable, SchemaVisitor

class _DDLCompiles(ClauseElement): ...
class DDLElement(Executable, _DDLCompiles): ...
class DDL(DDLElement): ...
class _CreateDropBase(DDLElement): ...
class CreateSchema(_CreateDropBase): ...
class DropSchema(_CreateDropBase): ...
class CreateTable(_CreateDropBase): ...
class _DropView(_CreateDropBase): ...
class CreateColumn(_DDLCompiles): ...
class DropTable(_CreateDropBase): ...
class CreateSequence(_CreateDropBase): ...
class DropSequence(_CreateDropBase): ...
class CreateIndex(_CreateDropBase): ...
class DropIndex(_CreateDropBase): ...
class AddConstraint(_CreateDropBase): ...
class DropConstraint(_CreateDropBase): ...
class DDLBase(SchemaVisitor): ...
class SchemaGenerator(DDLBase): ...
class SchemaDropper(DDLBase): ...

def sort_tables(tables, skip_fn=..., extra_dependencies=...): ...
def sort_tables_and_constraints(tables, filter_fn=..., extra_dependencies=...): ...



from typing import Any, Awaitable, TypeVar, List, Callable, Tuple, Union, Dict, Generator
from abc import ABCMeta, abstractmethod
from asyncio.futures import Future
from asyncio.coroutines import coroutine

__all__ = ... # type: str

_T = TypeVar('_T')

PIPE = ...  # type: Any  # from subprocess.PIPE

AF_UNSPEC = 0     # from socket
AI_PASSIVE = 0

class Handle:
    __slots__ = [] # type: List[str]
    _cancelled = False
    _args = [] # type: List[Any]
    def __init__(self, callback: Callable[[],Any], args: List[Any],
        loop: AbstractEventLoop) -> None: ...
    def __repr__(self) -> str: ...
    def cancel(self) -> None: ...
    def _run(self) -> None: ...

class AbstractServer:
    def close(self) -> None: ...
    @coroutine
    def wait_closed(self) -> None: ...

class AbstractEventLoop(metaclass=ABCMeta):
    @abstractmethod
    def run_forever(self) -> None: ...
    @abstractmethod
    def run_until_complete(self, future: Union[Awaitable[_T], Future[_T], Generator[Any, Any, _T]]) -> _T: ...
    @abstractmethod
    def stop(self) -> None: ...
    @abstractmethod
    def is_running(self) -> bool: ...
    @abstractmethod
    def close(self) -> None: ...
    # Methods scheduling callbacks.  All these return Handles.
    @abstractmethod
    def call_soon(self, callback: Callable[[],Any], *args: Any) -> Handle: ...
    @abstractmethod
    def call_later(self, delay: Union[int, float], callback: Callable[[],Any], *args: Any) -> Handle: ...
    @abstractmethod
    def call_at(self, when: float, callback: Callable[[],Any], *args: Any) -> Handle: ...
    @abstractmethod
    def time(self) -> float: ...
    # Methods for interacting with threads
    @abstractmethod
    def call_soon_threadsafe(self, callback: Callable[[],Any], *args: Any) -> Handle: ...
    @abstractmethod
    def run_in_executor(self, executor: Any,
        callback: Callable[[],Any], *args: Any) -> Future[Any]: ...
    @abstractmethod
    def set_default_executor(self, executor: Any) -> None: ...
    # Network I/O methods returning Futures.
    @abstractmethod
    def getaddrinfo(self, host: str, port: int, *,
        family: int = ..., type: int = ..., proto: int = ..., flags: int = ...) -> List[Tuple[int, int, int, str, tuple]]: ...
    @abstractmethod
    def getnameinfo(self, sockaddr: tuple, flags: int = ...) -> Tuple[str, int]: ...
    @abstractmethod
    def create_connection(self, protocol_factory: Any, host: str = ..., port: int = ..., *,
                          ssl: Any = ..., family: int = ..., proto: int = ..., flags: int = ..., sock: Any = ...,
                          local_addr: str = ..., server_hostname: str = ...) -> tuple: ...
                          # ?? check Any
                          # return (Transport, Protocol)
    @abstractmethod
    def create_server(self, protocol_factory: Any, host: str = ..., port: int = ..., *,
                      family: int = ..., flags: int = ...,
                      sock: Any = ..., backlog: int = ..., ssl: Any = ..., reuse_address: Any = ...) -> Any: ...
                    # ?? check Any
                    # return Server
    @abstractmethod
    def create_unix_connection(self, protocol_factory: Any, path: str, *,
                               ssl: Any = ..., sock: Any = ...,
                               server_hostname: str = ...) -> tuple: ...
                    # ?? check Any
                    # return tuple(Transport, Protocol)
    @abstractmethod
    def create_unix_server(self, protocol_factory: Any, path: str, *,
                           sock: Any = ..., backlog: int = ..., ssl: Any = ...) -> Any: ...
                    # ?? check Any
                    # return Server
    @abstractmethod
    def create_datagram_endpoint(self, protocol_factory: Any,
                                 local_addr: str = ..., remote_addr: str = ..., *,
                                 family: int = ..., proto: int = ..., flags: int = ...) -> tuple: ...
                    #?? check Any
                    # return (Transport, Protocol)
    # Pipes and subprocesses.
    @abstractmethod
    def connect_read_pipe(self, protocol_factory: Any, pipe: Any) -> tuple: ...
                    #?? check Any
                    # return (Transport, Protocol)
    @abstractmethod
    def connect_write_pipe(self, protocol_factory: Any, pipe: Any) -> tuple: ...
                    #?? check Any
                    # return (Transport, Protocol)
    @abstractmethod
    def subprocess_shell(self, protocol_factory: Any, cmd: Union[bytes, str], *, stdin: Any = ...,
                         stdout: Any = ..., stderr: Any = ...,
                         **kwargs: Dict[str, Any]) -> tuple: ...
                    #?? check Any
                    # return (Transport, Protocol)
    @abstractmethod
    def subprocess_exec(self, protocol_factory: Any, *args: List[Any], stdin: Any = ...,
                        stdout: Any = ..., stderr: Any = ...,
                        **kwargs: Dict[str, Any]) -> tuple: ...
                    #?? check Any
                    # return (Transport, Protocol)
    @abstractmethod
    def add_reader(self, fd: int, callback: Callable[[],Any], *args: List[Any]) -> None: ...
    @abstractmethod
    def remove_reader(self, fd: int) -> None: ...
    @abstractmethod
    def add_writer(self, fd: int, callback: Callable[[],Any], *args: List[Any]) -> None: ...
    @abstractmethod
    def remove_writer(self, fd: int) -> None: ...
    # Completion based I/O methods returning Futures.
    @abstractmethod
    def sock_recv(self, sock: Any, nbytes: int) -> Any: ... #TODO
    @abstractmethod
    def sock_sendall(self, sock: Any, data: bytes) -> None: ... #TODO
    @abstractmethod
    def sock_connect(self, sock: Any, address: str) -> Any: ... #TODO
    @abstractmethod
    def sock_accept(self, sock: Any) -> Any: ...
    # Signal handling.
    @abstractmethod
    def add_signal_handler(self, sig: int, callback: Callable[[],Any], *args: List[Any]) -> None: ...
    @abstractmethod
    def remove_signal_handler(self, sig: int) -> None: ...
    # Error handlers.
    @abstractmethod
    def set_exception_handler(self, handler: Callable[[], Any]) -> None: ...
    @abstractmethod
    def default_exception_handler(self, context: Any) -> None: ...
    @abstractmethod
    def call_exception_handler(self, context: Any) -> None: ...
    # Debug flag management.
    @abstractmethod
    def get_debug(self) -> bool: ...
    @abstractmethod
    def set_debug(self, enabled: bool) -> None: ...

class AbstractEventLoopPolicy(metaclass=ABCMeta):
    @abstractmethod
    def get_event_loop(self) -> AbstractEventLoop: ...
    @abstractmethod
    def set_event_loop(self, loop: AbstractEventLoop): ...
    @abstractmethod
    def new_event_loop(self) -> Any: ... # return selector_events.BaseSelectorEventLoop
    # Child processes handling (Unix only).
    @abstractmethod
    def get_child_watcher(self) -> Any: ...  # return unix_events.AbstractChildWatcher
    @abstractmethod
    def set_child_watcher(self, watcher: Any) -> None: ... # gen unix_events.AbstractChildWatcher

class BaseDefaultEventLoopPolicy(AbstractEventLoopPolicy):
    def __init__(self) -> None: ...
    def get_event_loop(self) -> AbstractEventLoop: ...
    def set_event_loop(self, loop: AbstractEventLoop): ...
    def new_event_loop(self) -> Any: ... # Same return than AbstractEventLoop


def get_event_loop() -> AbstractEventLoop: ...



# Stubs for requests.packages.urllib3.packages.ssl_match_hostname._implementation (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

class CertificateError(ValueError): ...

def match_hostname(cert, hostname): ...



from typing import List

# In Python, arguments have integer default values
def spawn(cmd: List[str], search_path: bool = ..., verbose: bool = ...,
           dry_run: bool = ...) -> None: ...
def find_executable(executable: str, path: str = ...) -> str: ...



"""Stubs for the 'unicodedata' module."""

from typing import Any, TypeVar, Union

ucd_3_2_0 = ... # type: UCD
unidata_version = ... # type: str
# PyCapsule
ucnhash_CAPI = ... # type: Any

_default = TypeVar("_default")

def bidirectional(unichr: unicode) -> str: ...
def category(unichr: unicode) -> str: ...
def combining(unichr: unicode) -> int: ...
def decimal(chr: unicode, default: _default = ...) -> Union[int, _default]: ...
def decomposition(unichr: unicode) -> str: ...
def digit(chr: unicode, default: _default = ...) -> Union[int, _default]: ...
def east_asian_width(unichr: unicode) -> str: ...
def lookup(name: str) -> unicode: ...
def mirrored(unichr: unicode) -> int: ...
def name(chr: unicode, default: _default = ...) -> Union[str, _default]: ...
def normalize(form: str, unistr: unicode) -> unicode: ...
def numeric(chr, default: _default = ...) -> Union[float, _default]: ...

class UCD(object):
    unidata_version = ... # type: str
    # The methods below are constructed from the same array in C
    # (unicodedata_functions) and hence identical to the methods above.
    def bidirectional(self, unichr: unicode) -> str: ...
    def category(self, unichr: unicode) -> str: ...
    def combining(self, unichr: unicode) -> int: ...
    def decimal(self, chr: unicode, default: _default = ...) -> Union[int, _default]: ...
    def decomposition(self, unichr: unicode) -> str: ...
    def digit(self, chr: unicode, default: _default = ...) -> Union[int, _default]: ...
    def east_asian_width(self, unichr: unicode) -> str: ...
    def lookup(self, name: str) -> unicode: ...
    def mirrored(self, unichr: unicode) -> int: ...
    def name(self, chr: unicode, default: _default = ...) -> Union[str, _default]: ...
    def normalize(self, form: str, unistr: unicode) -> unicode: ...
    def numeric(self, chr: unicode, default: _default = ...) -> Union[float, _default]: ...



# Stubs for http.client (Python 3.4)

from typing import Any, Dict
import email.message
import io

responses = ...  # type: Dict[int, str]

class HTTPMessage(email.message.Message):
    def getallmatchingheaders(self, name): ...

class HTTPResponse(io.RawIOBase):
    fp = ...  # type: Any
    debuglevel = ...  # type: Any
    headers = ...  # type: Any
    version = ...  # type: Any
    status = ...  # type: Any
    reason = ...  # type: Any
    chunked = ...  # type: Any
    chunk_left = ...  # type: Any
    length = ...  # type: Any
    will_close = ...  # type: Any
    def __init__(self, sock, debuglevel=..., method=..., url=...) -> None: ...
    code = ...  # type: Any
    def begin(self): ...
    def close(self): ...
    def flush(self): ...
    def readable(self): ...
    def isclosed(self): ...
    def read(self, amt=...): ...
    def readinto(self, b): ...
    def fileno(self): ...
    def getheader(self, name, default=...): ...
    def getheaders(self): ...
    def __iter__(self): ...
    def info(self): ...
    def geturl(self): ...
    def getcode(self): ...

class HTTPConnection:
    response_class = ...  # type: Any
    default_port = ...  # type: Any
    auto_open = ...  # type: Any
    debuglevel = ...  # type: Any
    mss = ...  # type: Any
    timeout = ...  # type: Any
    source_address = ...  # type: Any
    sock = ...  # type: Any
    def __init__(self, host, port=..., timeout=..., source_address=...) -> None: ...
    def set_tunnel(self, host, port=..., headers=...): ...
    def set_debuglevel(self, level): ...
    def connect(self): ...
    def close(self): ...
    def send(self, data): ...
    def putrequest(self, method, url, skip_host=..., skip_accept_encoding=...): ...
    def putheader(self, header, *values): ...
    def endheaders(self, message_body=...): ...
    def request(self, method, url, body=..., headers=...): ...
    def getresponse(self): ...

class HTTPSConnection(HTTPConnection):
    default_port = ...  # type: Any
    key_file = ...  # type: Any
    cert_file = ...  # type: Any
    def __init__(self, host, port=..., key_file=..., cert_file=..., timeout=...,
                 source_address=..., *, context=..., check_hostname=...): ...
    sock = ...  # type: Any
    def connect(self): ...

class HTTPException(Exception): ...
class NotConnected(HTTPException): ...
class InvalidURL(HTTPException): ...

class UnknownProtocol(HTTPException):
    args = ...  # type: Any
    version = ...  # type: Any
    def __init__(self, version) -> None: ...

class UnknownTransferEncoding(HTTPException): ...
class UnimplementedFileMode(HTTPException): ...

class IncompleteRead(HTTPException):
    args = ...  # type: Any
    partial = ...  # type: Any
    expected = ...  # type: Any
    def __init__(self, partial, expected=...) -> None: ...

class ImproperConnectionState(HTTPException): ...
class CannotSendRequest(ImproperConnectionState): ...
class CannotSendHeader(ImproperConnectionState): ...
class ResponseNotReady(ImproperConnectionState): ...

class BadStatusLine(HTTPException):
    args = ...  # type: Any
    line = ...  # type: Any
    def __init__(self, line) -> None: ...

class LineTooLong(HTTPException):
    def __init__(self, line_type) -> None: ...

error = HTTPException



"""Mypy parser.

Constructs a parse tree (abstract syntax tree) based on a string
representing a source file. Performs only minimal semantic checks.
"""

import re

from typing import List, Tuple, Any, Set, cast, Union, Optional

from mypy import lex, docstring
from mypy.lex import (
    Token, Eof, Bom, Break, Name, Colon, Dedent, IntLit, StrLit, BytesLit,
    UnicodeLit, FloatLit, Op, Indent, Keyword, Punct, LexError, ComplexLit,
    EllipsisToken
)
import mypy.types
from mypy.nodes import (
    MypyFile, Import, Node, ImportAll, ImportFrom, FuncDef, OverloadedFuncDef,
    ClassDef, Decorator, Block, Var, OperatorAssignmentStmt,
    ExpressionStmt, AssignmentStmt, ReturnStmt, RaiseStmt, AssertStmt,
    DelStmt, BreakStmt, ContinueStmt, PassStmt, GlobalDecl,
    WhileStmt, ForStmt, IfStmt, TryStmt, WithStmt,
    TupleExpr, GeneratorExpr, ListComprehension, ListExpr, ConditionalExpr,
    DictExpr, SetExpr, NameExpr, IntExpr, StrExpr, BytesExpr, UnicodeExpr,
    FloatExpr, CallExpr, SuperExpr, MemberExpr, IndexExpr, SliceExpr, OpExpr,
    UnaryExpr, FuncExpr, PrintStmt, ImportBase, ComparisonExpr,
    StarExpr, YieldFromExpr, NonlocalDecl, DictionaryComprehension,
    SetComprehension, ComplexExpr, EllipsisExpr, YieldExpr, ExecStmt, Argument,
    BackquoteExpr
)
from mypy import defaults
from mypy import nodes
from mypy.errors import Errors, CompileError
from mypy.types import Type, CallableType, AnyType, UnboundType
from mypy.parsetype import (
    parse_type, parse_types, parse_signature, TypeParseError, parse_str_as_signature
)


precedence = {
    '**': 16,
    '-u': 15, '+u': 15, '~': 15,   # unary operators (-, + and ~)
    '<cast>': 14,
    '*': 13, '/': 13, '//': 13, '%': 13,
    '+': 12, '-': 12,
    '>>': 11, '<<': 11,
    '&': 10,
    '^': 9,
    '|': 8,
    '==': 7, '!=': 7, '<': 7, '>': 7, '<=': 7, '>=': 7, 'is': 7, 'in': 7,
    '*u': 7,  # unary * for star expressions
    'not': 6,
    'and': 5,
    'or': 4,
    '<if>': 3,  # conditional expression
    '<for>': 2,  # list comprehension
    ',': 1}


op_assign = set([
    '+=', '-=', '*=', '/=', '//=', '%=', '**=', '|=', '&=', '^=', '>>=',
    '<<='])

op_comp = set([
    '>', '<', '==', '>=', '<=', '<>', '!=', 'is', 'is', 'in', 'not'])

none = Token('')  # Empty token


def parse(source: Union[str, bytes], fnam: str = None, errors: Errors = None,
          pyversion: Tuple[int, int] = defaults.PYTHON3_VERSION,
          custom_typing_module: str = None,
          fast_parser: bool = False) -> MypyFile:
    """Parse a source file, without doing any semantic analysis.

    Return the parse tree. If errors is not provided, raise ParseError
    on failure. Otherwise, use the errors object to report parse errors.

    The pyversion (major, minor) argument determines the Python syntax variant.
    """
    if fast_parser:
        import mypy.fastparse
        return mypy.fastparse.parse(source,
                                    fnam=fnam,
                                    errors=errors,
                                    pyversion=pyversion,
                                    custom_typing_module=custom_typing_module)

    is_stub_file = bool(fnam) and fnam.endswith('.pyi')
    parser = Parser(fnam,
                    errors,
                    pyversion,
                    custom_typing_module,
                    is_stub_file=is_stub_file)
    tree = parser.parse(source)
    tree.path = fnam
    tree.is_stub = is_stub_file
    return tree


class Parser:
    """Mypy parser that parses a string into an AST.

    Parses type annotations in addition to basic Python syntax. It supports both Python 2 and 3
    (though Python 2 support is incomplete).

    The AST classes are defined in mypy.nodes and mypy.types.
    """

    tok = None  # type: List[Token]
    ind = 0
    errors = None  # type: Errors
    # If True, raise an exception on any parse error. Otherwise, errors are reported via 'errors'.
    raise_on_error = False

    # Are we currently parsing the body of a class definition?
    is_class_body = False
    # All import nodes encountered so far in this parse unit.
    imports = None  # type: List[ImportBase]
    # Names imported from __future__.
    future_options = None  # type: List[str]
    # Lines to ignore (using # type: ignore).
    ignored_lines = None  # type: Set[int]

    def __init__(self, fnam: str, errors: Errors, pyversion: Tuple[int, int],
                 custom_typing_module: str = None, is_stub_file: bool = False) -> None:
        self.raise_on_error = errors is None
        self.pyversion = pyversion
        self.custom_typing_module = custom_typing_module
        self.is_stub_file = is_stub_file
        if errors is not None:
            self.errors = errors
        else:
            self.errors = Errors()
        if fnam is not None:
            self.errors.set_file(fnam)
        else:
            self.errors.set_file('<input>')

    def parse(self, s: Union[str, bytes]) -> MypyFile:
        self.tok, self.ignored_lines = lex.lex(s, pyversion=self.pyversion,
                                               is_stub_file=self.is_stub_file)
        self.ind = 0
        self.imports = []
        self.future_options = []
        file = self.parse_file()
        if self.raise_on_error and self.errors.is_errors():
            self.errors.raise_error()
        return file

    def weak_opts(self) -> Set[str]:
        """Do weak typing if any of the first ten tokens is a comment saying so.

        The comment can be one of:
        # mypy: weak=global
        # mypy: weak=local
        # mypy: weak      <- defaults to local
        """
        regexp = re.compile(r'^[\s]*# *mypy: *weak(=?)([^\s]*)', re.M)
        for t in self.tok[:10]:
            for s in [t.string, t.pre]:
                m = regexp.search(s)
                if m:
                    opts = set(x for x in m.group(2).split(',') if x)
                    if not opts:
                        opts.add('local')
                    return opts
        return set()

    def parse_file(self) -> MypyFile:
        """Parse a mypy source file."""
        is_bom = self.parse_bom()
        defs = self.parse_defs()
        weak_opts = self.weak_opts()
        self.expect_type(Eof)
        # Skip imports that have been ignored (so that we can ignore a C extension module without
        # stub, for example), except for 'from x import *', because we wouldn't be able to
        # determine which names should be defined unless we process the module. We can still
        # ignore errors such as redefinitions when using the latter form.
        imports = [node for node in self.imports
                   if node.line not in self.ignored_lines or isinstance(node, ImportAll)]
        node = MypyFile(defs, imports, is_bom, self.ignored_lines,
                        weak_opts=weak_opts)
        return node

    # Parse the initial part

    def parse_bom(self) -> bool:
        """Parse the optional byte order mark at the beginning of a file."""
        if isinstance(self.current(), Bom):
            self.expect_type(Bom)
            if isinstance(self.current(), Break):
                self.expect_break()
            return True
        else:
            return False

    def parse_import(self) -> Import:
        self.expect('import')
        ids = []
        while True:
            id = self.parse_qualified_name()
            translated = self.translate_module_id(id)
            as_id = None
            if self.current_str() == 'as':
                self.expect('as')
                name_tok = self.expect_type(Name)
                as_id = name_tok.string
            elif translated != id:
                as_id = id
            ids.append((translated, as_id))
            if self.current_str() != ',':
                break
            self.expect(',')
        node = Import(ids)
        self.imports.append(node)
        return node

    def translate_module_id(self, id: str) -> str:
        """Return the actual, internal module id for a source text id.

        For example, translate '__builtin__' in Python 2 to 'builtins'.
        """
        if id == self.custom_typing_module:
            return 'typing'
        elif id == '__builtin__' and self.pyversion[0] == 2:
            # HACK: __builtin__ in Python 2 is aliases to builtins. However, the implementation
            #   is named __builtin__.py (there is another layer of translation elsewhere).
            return 'builtins'
        return id

    def parse_import_from(self) -> Node:
        self.expect('from')

        # Build the list of beginning relative tokens.
        relative = 0
        while self.current_str() in (".", "..."):
            relative += len(self.current_str())
            self.skip()

        # Parse qualified name to actually import from.
        if self.current_str() == "import":
            # Empty/default values.
            name = ""
        else:
            name = self.parse_qualified_name()

        name = self.translate_module_id(name)

        # Parse import list
        self.expect('import')
        node = None  # type: ImportBase
        if self.current_str() == '*':
            if name == '__future__':
                self.parse_error()
            # An import all from a module node:
            self.skip()
            node = ImportAll(name, relative)
        else:
            is_paren = self.current_str() == '('
            if is_paren:
                self.expect('(')
            targets = []  # type: List[Tuple[str, str]]
            while True:
                id, as_id = self.parse_import_name()
                if '%s.%s' % (name, id) == self.custom_typing_module:
                    if targets or self.current_str() == ',':
                        self.fail('You cannot import any other modules when you '
                                  'import a custom typing module',
                                  self.current().line)
                    node = Import([('typing', as_id)])
                    self.skip_until_break()
                    break
                targets.append((id, as_id))
                if self.current_str() != ',':
                    break
                self.expect(',')
                if is_paren and self.current_str() == ')':
                    break
            if is_paren:
                self.expect(')')
            if node is None:
                node = ImportFrom(name, relative, targets)
        self.imports.append(node)
        if name == '__future__':
            self.future_options.extend(target[0] for target in targets)
        return node

    def parse_import_name(self) -> Tuple[str, Optional[str]]:
        tok = self.expect_type(Name)
        name = tok.string
        if self.current_str() == 'as':
            self.skip()
            as_name = self.expect_type(Name)
            return name, as_name.string
        else:
            return name, None

    def parse_qualified_name(self) -> str:
        """Parse a name with an optional module qualifier.

        Return a tuple with the name as a string and a token array
        containing all the components of the name.
        """
        tok = self.expect_type(Name)
        n = tok.string
        while self.current_str() == '.':
            self.expect('.')
            tok = self.expect_type(Name)
            n += '.' + tok.string
        return n

    # Parsing global definitions

    def parse_defs(self) -> List[Node]:
        defs = []  # type: List[Node]
        while not self.eof():
            try:
                defn, is_simple = self.parse_statement()
                if is_simple:
                    self.expect_break()
                if defn is not None:
                    if not self.try_combine_overloads(defn, defs):
                        defs.append(defn)
            except ParseError:
                pass
        return defs

    def parse_class_def(self) -> ClassDef:
        old_is_class_body = self.is_class_body
        self.is_class_body = True

        self.expect('class')
        metaclass = None

        try:
            base_types = []  # type: List[Node]
            try:
                name_tok = self.expect_type(Name)
                name = name_tok.string

                self.errors.push_type(name)

                if self.current_str() == '(':
                    self.skip()
                    while True:
                        if self.current_str() == ')':
                            break
                        if self.peek().string == '=':
                            metaclass = self.parse_class_keywords()
                            break
                        base_types.append(self.parse_super_type())
                        if self.current_str() != ',':
                            break
                        self.skip()
                    self.expect(')')
            except ParseError:
                pass

            defs, _ = self.parse_block()

            node = ClassDef(name, defs, None, base_types, metaclass=metaclass)
            return node
        finally:
            self.errors.pop_type()
            self.is_class_body = old_is_class_body

    def parse_class_keywords(self) -> Optional[str]:
        """Parse the class keyword arguments, keeping the metaclass but
        ignoring all others.  Returns None if the metaclass isn't found.
        """
        metaclass = None
        while True:
            key = self.expect_type(Name)
            self.expect('=')
            if key.string == 'metaclass':
                metaclass = self.parse_qualified_name()
            else:
                # skip the class value
                self.parse_expression(precedence[','])
            if self.current_str() != ',':
                break
            self.skip()
            if self.current_str() == ')':
                break
        return metaclass

    def parse_super_type(self) -> Node:
        return self.parse_expression(precedence[','])

    def parse_decorated_function_or_class(self) -> Node:
        decorators = []
        no_type_checks = False
        while self.current_str() == '@':
            self.expect('@')
            d_exp = self.parse_expression()
            if self.is_no_type_check_decorator(d_exp):
                no_type_checks = True
            decorators.append(d_exp)
            self.expect_break()
        if self.current_str() != 'class':
            func = self.parse_function(no_type_checks)
            func.is_decorated = True
            var = Var(func.name())
            # Types of decorated functions must always be inferred.
            var.is_ready = False
            var.set_line(decorators[0].line)
            node = Decorator(func, decorators, var)
            return node
        else:
            cls = self.parse_class_def()
            cls.decorators = decorators
            return cls

    def is_no_type_check_decorator(self, expr: Node) -> bool:
        if isinstance(expr, NameExpr):
            return expr.name == 'no_type_check'
        elif isinstance(expr, MemberExpr):
            if isinstance(expr.expr, NameExpr):
                return expr.expr.name == 'typing' and expr.name == 'no_type_check'
        else:
            return False

    def parse_function(self, no_type_checks: bool=False) -> FuncDef:
        def_tok = self.expect('def')
        is_method = self.is_class_body
        self.is_class_body = False
        try:
            (name, args, typ, is_error, extra_stmts) = self.parse_function_header(no_type_checks)

            arg_kinds = [arg.kind for arg in args]
            arg_names = [arg.variable.name() for arg in args]

            body, comment_type = self.parse_block(allow_type=True)
            # Potentially insert extra assignment statements to the beginning of the
            # body, used to decompose Python 2 tuple arguments.
            body.body[:0] = extra_stmts
            if comment_type:
                # The function has a # type: ... signature.
                if typ:
                    self.errors.report(
                        def_tok.line, 'Function has duplicate type signatures')
                sig = cast(CallableType, comment_type)
                if sig.is_ellipsis_args:
                    # When we encounter an ellipsis, fill in the arg_types with
                    # a bunch of AnyTypes, emulating Callable[..., T]
                    arg_types = [AnyType()] * len(arg_kinds)  # type: List[Type]
                    typ = CallableType(
                        arg_types,
                        arg_kinds,
                        arg_names,
                        sig.ret_type,
                        None,
                        line=def_tok.line)
                elif is_method and len(sig.arg_kinds) < len(arg_kinds):
                    self.check_argument_kinds(arg_kinds,
                                              [nodes.ARG_POS] + sig.arg_kinds,
                                              def_tok.line)
                    # Add implicit 'self' argument to signature.
                    first_arg = [AnyType()]  # type: List[Type]
                    typ = CallableType(
                        first_arg + sig.arg_types,
                        arg_kinds,
                        arg_names,
                        sig.ret_type,
                        None,
                        line=def_tok.line)
                else:
                    self.check_argument_kinds(arg_kinds, sig.arg_kinds,
                                              def_tok.line)
                    typ = CallableType(
                        sig.arg_types,
                        arg_kinds,
                        arg_names,
                        sig.ret_type,
                        None,
                        line=def_tok.line)

            # If there was a serious error, we really cannot build a parse tree
            # node.
            if is_error:
                return None

            node = FuncDef(name, args, body, typ)
            node.set_line(def_tok)
            if typ is not None:
                typ.definition = node
            return node
        finally:
            self.errors.pop_function()
            self.is_class_body = is_method

    def check_argument_kinds(self, funckinds: List[int], sigkinds: List[int],
                             line: int) -> None:
        """Check that arguments are consistent.

        This verifies that they have the same number and the kinds correspond.

        Arguments:
          funckinds: kinds of arguments in function definition
          sigkinds:  kinds of arguments in signature (after # type:)
        """
        if len(funckinds) != len(sigkinds):
            if len(funckinds) > len(sigkinds):
                self.fail("Type signature has too few arguments", line)
            else:
                self.fail("Type signature has too many arguments", line)
            return
        for kind, token in [(nodes.ARG_STAR, '*'),
                            (nodes.ARG_STAR2, '**')]:
            if ((funckinds.count(kind) != sigkinds.count(kind)) or
                    (kind in funckinds and sigkinds.index(kind) != funckinds.index(kind))):
                self.fail(
                    "Inconsistent use of '{}' in function "
                    "signature".format(token), line)

    def parse_function_header(
            self, no_type_checks: bool=False) -> Tuple[str,
                                                       List[Argument],
                                                       CallableType,
                                                       bool,
                                                       List[AssignmentStmt]]:
        """Parse function header (a name followed by arguments)

        Return a 5-tuple with the following items:
          name
          arguments
          signature (annotation)
          error flag (True if error)
          extra statements needed to decompose arguments (usually empty)

        See parse_arg_list for an explanation of the final tuple item.
        """
        name = ''

        try:
            name_tok = self.expect_type(Name)
            name = name_tok.string

            self.errors.push_function(name)

            args, typ, extra_stmts = self.parse_args(no_type_checks)
        except ParseError:
            if not isinstance(self.current(), Break):
                self.ind -= 1  # Kludge: go back to the Break token
            # Resynchronise parsing by going back over :, if present.
            if isinstance(self.tok[self.ind - 1], Colon):
                self.ind -= 1
            return (name, [], None, True, [])

        return (name, args, typ, False, extra_stmts)

    def parse_args(self, no_type_checks: bool=False) -> Tuple[List[Argument],
                                                              CallableType,
                                                              List[AssignmentStmt]]:
        """Parse a function signature (...) [-> t].

        See parse_arg_list for an explanation of the final tuple item.
        """
        lparen = self.expect('(')

        # Parse the argument list (everything within '(' and ')').
        args, extra_stmts = self.parse_arg_list(no_type_checks=no_type_checks)

        self.expect(')')

        if self.current_str() == '->':
            self.skip()
            if no_type_checks:
                self.parse_expression()
                ret_type = None
            else:
                ret_type = self.parse_type()
        else:
            ret_type = None

        arg_kinds = [arg.kind for arg in args]
        self.verify_argument_kinds(arg_kinds, lparen.line)

        annotation = self.build_func_annotation(
            ret_type, args, lparen.line)

        return args, annotation, extra_stmts

    def build_func_annotation(self, ret_type: Type, args: List[Argument],
            line: int, is_default_ret: bool = False) -> CallableType:
        arg_types = [arg.type_annotation for arg in args]
        # Are there any type annotations?
        if ((ret_type and not is_default_ret)
                or arg_types != [None] * len(arg_types)):
            # Yes. Construct a type for the function signature.
            return self.construct_function_type(args, ret_type, line)
        else:
            return None

    def parse_arg_list(self, allow_signature: bool = True,
            no_type_checks: bool=False) -> Tuple[List[Argument],
                                                 List[AssignmentStmt]]:
        """Parse function definition argument list.

        This includes everything between '(' and ')' (but not the
        parentheses).

        Return tuple (arguments,
                      extra statements for decomposing arguments).

        The final argument is only used for Python 2 argument lists with
        tuples; they contain destructuring assignment statements used to
        decompose tuple arguments. For example, consider a header like this:

        . def f((x, y))

        The actual (sole) argument will be __tuple_arg_1 (a generated
        name), whereas the extra statement list will contain a single
        assignment statement corresponding to this assignment:

          x, y = __tuple_arg_1
        """
        args = []  # type: List[Argument]
        extra_stmts = []
        # This is for checking duplicate argument names.
        arg_names = []  # type: List[str]
        has_tuple_arg = False

        require_named = False
        bare_asterisk_before = -1

        if self.current_str() != ')' and self.current_str() != ':':
            while self.current_str() != ')':
                if self.current_str() == '*' and self.peek().string == ',':
                    self.expect('*')
                    require_named = True
                    bare_asterisk_before = len(args)
                elif self.current_str() in ['*', '**']:
                    if bare_asterisk_before == len(args):
                        # named arguments must follow bare *
                        self.parse_error()

                    arg = self.parse_asterisk_arg(
                        allow_signature,
                        no_type_checks,
                    )
                    args.append(arg)
                    require_named = True
                elif self.current_str() == '(':
                    arg, extra_stmt, names = self.parse_tuple_arg(len(args))
                    args.append(arg)
                    if extra_stmt is not None:
                        extra_stmts.append(extra_stmt)
                        has_tuple_arg = True
                    arg_names.extend(names)
                else:
                    arg, require_named = self.parse_normal_arg(
                        require_named,
                        allow_signature,
                        no_type_checks,
                    )
                    args.append(arg)
                    arg_names.append(arg.variable.name())

                if self.current().string != ',':
                    break

                self.expect(',')

        # Non-tuple argument dupes will be checked elsewhere. Avoid
        # generating duplicate errors.
        if has_tuple_arg:
            self.check_duplicate_argument_names(arg_names)

        return args, extra_stmts

    def check_duplicate_argument_names(self, names: List[str]) -> None:
        found = set()  # type: Set[str]
        for name in names:
            if name in found:
                self.fail('Duplicate argument name "{}"'.format(name),
                          self.current().line)
            found.add(name)

    def parse_asterisk_arg(self,
            allow_signature: bool,
            no_type_checks: bool) -> Argument:
        asterisk = self.skip()
        name = self.expect_type(Name)
        variable = Var(name.string)
        if asterisk.string == '*':
            kind = nodes.ARG_STAR
        else:
            kind = nodes.ARG_STAR2

        type = None
        if no_type_checks:
            self.parse_parameter_annotation()
        else:
            type = self.parse_arg_type(allow_signature)

        return Argument(variable, type, None, kind)

    def parse_tuple_arg(self, index: int) -> Tuple[Argument, AssignmentStmt, List[str]]:
        """Parse a single Python 2 tuple argument.

        Example: def f(x, (y, z)): ...

        The tuple arguments gets transformed into an assignment in the
        function body (the second return value).

        Return tuple (argument, decomposing assignment, list of names defined).

        Special case: if the argument is just (x) then it's not a tuple;
        we indicate this by returning (argument, None, ['x']).
        However, if the argument is (x,) then it *is* a (singleton) tuple.
        """
        line = self.current().line
        # Generate a new argument name that is very unlikely to clash with anything.
        arg_name = '__tuple_arg_{}'.format(index + 1)
        if self.pyversion[0] >= 3:
            self.fail('Tuples in argument lists only supported in Python 2 mode', line)
        paren_arg = self.parse_parentheses()
        self.verify_tuple_arg(paren_arg)
        if isinstance(paren_arg, NameExpr):
            # This isn't a tuple. Revert to a normal argument.
            arg_name = paren_arg.name
            decompose = None
        else:
            rvalue = NameExpr(arg_name)
            rvalue.set_line(line)
            decompose = AssignmentStmt([paren_arg], rvalue)
            decompose.set_line(line)
        kind = nodes.ARG_POS
        initializer = None
        if self.current_str() == '=':
            self.expect('=')
            initializer = self.parse_expression(precedence[','])
            kind = nodes.ARG_OPT
        var = Var(arg_name)
        arg_names = self.find_tuple_arg_argument_names(paren_arg)
        return Argument(var, None, initializer, kind), decompose, arg_names

    def verify_tuple_arg(self, paren_arg: Node) -> None:
        if isinstance(paren_arg, TupleExpr):
            if not paren_arg.items:
                self.fail('Empty tuple not valid as an argument', paren_arg.line)
            for item in paren_arg.items:
                self.verify_tuple_arg(item)
        elif not isinstance(paren_arg, NameExpr):
            self.fail('Invalid item in tuple argument', paren_arg.line)

    def find_tuple_arg_argument_names(self, node: Node) -> List[str]:
        result = []  # type: List[str]
        if isinstance(node, TupleExpr):
            for item in node.items:
                result.extend(self.find_tuple_arg_argument_names(item))
        elif isinstance(node, NameExpr):
            result.append(node.name)
        return result

    def parse_normal_arg(self, require_named: bool,
            allow_signature: bool,
            no_type_checks: bool) -> Tuple[Argument, bool]:
        name = self.expect_type(Name)
        variable = Var(name.string)

        type = None
        if no_type_checks:
            self.parse_parameter_annotation()
        else:
            type = self.parse_arg_type(allow_signature)

        initializer = None  # type: Node
        if self.current_str() == '=':
            self.expect('=')
            initializer = self.parse_expression(precedence[','])
            if require_named:
                kind = nodes.ARG_NAMED
            else:
                kind = nodes.ARG_OPT
        else:
            if require_named:
                kind = nodes.ARG_NAMED
            else:
                kind = nodes.ARG_POS

        return Argument(variable, type, initializer, kind), require_named

    def parse_parameter_annotation(self) -> Node:
        if self.current_str() == ':':
            self.skip()
            return self.parse_expression(precedence[','])

    def parse_arg_type(self, allow_signature: bool) -> Type:
        if self.current_str() == ':' and allow_signature:
            self.skip()
            return self.parse_type()
        else:
            return None

    def verify_argument_kinds(self, kinds: List[int], line: int) -> None:
        found = set()  # type: Set[int]
        for i, kind in enumerate(kinds):
            if kind == nodes.ARG_POS and found & set([nodes.ARG_OPT,
                                                      nodes.ARG_STAR,
                                                      nodes.ARG_STAR2]):
                self.fail('Invalid argument list', line)
            elif kind == nodes.ARG_STAR and nodes.ARG_STAR in found:
                self.fail('Invalid argument list', line)
            elif kind == nodes.ARG_STAR2 and i != len(kinds) - 1:
                self.fail('Invalid argument list', line)
            found.add(kind)

    def construct_function_type(self, args: List[Argument], ret_type: Type,
                                line: int) -> CallableType:
        # Complete the type annotation by replacing omitted types with 'Any'.
        arg_types = [arg.type_annotation for arg in args]
        for i in range(len(arg_types)):
            if arg_types[i] is None:
                arg_types[i] = AnyType(implicit=True)
        if ret_type is None:
            ret_type = AnyType(implicit=True)
        arg_kinds = [arg.kind for arg in args]
        arg_names = [arg.variable.name() for arg in args]
        return CallableType(arg_types, arg_kinds, arg_names, ret_type, None, name=None,
                        variables=None, line=line)

    # Parsing statements

    def parse_block(self, allow_type: bool = False) -> Tuple[Block, Type]:
        colon = self.expect(':')
        if not isinstance(self.current(), Break):
            # Block immediately after ':'.
            nodes = []
            while True:
                ind = self.ind
                stmt, is_simple = self.parse_statement()
                if not is_simple:
                    self.parse_error_at(self.tok[ind])
                    break
                nodes.append(stmt)
                brk = self.expect_break()
                if brk.string != ';':
                    break
            node = Block(nodes)
            node.set_line(colon)
            return node, None
        else:
            # Indented block.
            brk = self.expect_break()
            type = self.parse_type_comment(brk, signature=True)
            self.expect_indent()
            stmt_list = []  # type: List[Node]
            if allow_type:
                cur = self.current()
                if type is None and isinstance(cur, StrLit):
                    ds = docstring.parse_docstring(cast(StrLit, cur).parsed())
                    if ds and False:  # TODO: Enable when this is working.
                        try:
                            type = parse_str_as_signature(ds.as_type_str(), cur.line)
                        except TypeParseError:
                            # We don't require docstrings to be actually correct.
                            # TODO: Report something here.
                            type = None
            while (not isinstance(self.current(), Dedent) and
                   not isinstance(self.current(), Eof)):
                try:
                    stmt, is_simple = self.parse_statement()
                    if is_simple:
                        self.expect_break()
                    if stmt is not None:
                        if not self.try_combine_overloads(stmt, stmt_list):
                            stmt_list.append(stmt)
                except ParseError:
                    pass
            if isinstance(self.current(), Dedent):
                self.skip()
            node = Block(stmt_list)
            node.set_line(colon)
            return node, type

    def try_combine_overloads(self, s: Node, stmt: List[Node]) -> bool:
        if isinstance(s, Decorator) and stmt:
            fdef = cast(Decorator, s)
            n = fdef.func.name()
            if (isinstance(stmt[-1], Decorator) and
                    (cast(Decorator, stmt[-1])).func.name() == n):
                stmt[-1] = OverloadedFuncDef([cast(Decorator, stmt[-1]), fdef])
                return True
            elif (isinstance(stmt[-1], OverloadedFuncDef) and
                    (cast(OverloadedFuncDef, stmt[-1])).name() == n):
                (cast(OverloadedFuncDef, stmt[-1])).items.append(fdef)
                return True
        return False

    def parse_statement(self) -> Tuple[Node, bool]:
        stmt = None  # type: Node
        t = self.current()
        ts = self.current_str()
        is_simple = True  # Is this a non-block statement?
        if ts == 'if':
            stmt = self.parse_if_stmt()
            is_simple = False
        elif ts == 'def':
            stmt = self.parse_function()
            is_simple = False
        elif ts == 'while':
            stmt = self.parse_while_stmt()
            is_simple = False
        elif ts == 'return':
            stmt = self.parse_return_stmt()
        elif ts == 'for':
            stmt = self.parse_for_stmt()
            is_simple = False
        elif ts == 'try':
            stmt = self.parse_try_stmt()
            is_simple = False
        elif ts == 'break':
            stmt = self.parse_break_stmt()
        elif ts == 'continue':
            stmt = self.parse_continue_stmt()
        elif ts == 'pass':
            stmt = self.parse_pass_stmt()
        elif ts == 'raise':
            stmt = self.parse_raise_stmt()
        elif ts == 'import':
            stmt = self.parse_import()
        elif ts == 'from':
            stmt = self.parse_import_from()
        elif ts == 'class':
            stmt = self.parse_class_def()
            is_simple = False
        elif ts == 'global':
            stmt = self.parse_global_decl()
        elif ts == 'nonlocal' and self.pyversion[0] >= 3:
            stmt = self.parse_nonlocal_decl()
        elif ts == 'assert':
            stmt = self.parse_assert_stmt()
        elif ts == 'del':
            stmt = self.parse_del_stmt()
        elif ts == 'with':
            stmt = self.parse_with_stmt()
            is_simple = False
        elif ts == '@':
            stmt = self.parse_decorated_function_or_class()
            is_simple = False
        elif ts == 'print' and (self.pyversion[0] == 2 and
                                'print_function' not in self.future_options):
            stmt = self.parse_print_stmt()
        elif ts == 'exec' and self.pyversion[0] == 2:
            stmt = self.parse_exec_stmt()
        else:
            stmt = self.parse_expression_or_assignment()
        if stmt is not None:
            stmt.set_line(t)
        return stmt, is_simple

    def parse_expression_or_assignment(self) -> Node:
        expr = self.parse_expression(star_expr_allowed=True)
        if self.current_str() == '=':
            return self.parse_assignment(expr)
        elif self.current_str() in op_assign:
            # Operator assignment statement.
            op = self.current_str()[:-1]
            self.skip()
            rvalue = self.parse_expression()
            return OperatorAssignmentStmt(op, expr, rvalue)
        else:
            # Expression statement.
            return ExpressionStmt(expr)

    def parse_assignment(self, lvalue: Any) -> Node:
        """Parse an assignment statement.

        Assume that lvalue has been parsed already, and the current token is '='.
        Also parse an optional '# type:' comment.
        """
        self.expect('=')
        lvalues = [lvalue]
        expr = self.parse_expression(star_expr_allowed=True)
        while self.current_str() == '=':
            self.skip()
            lvalues.append(expr)
            expr = self.parse_expression(star_expr_allowed=True)
        cur = self.current()
        if isinstance(cur, Break):
            type = self.parse_type_comment(cur, signature=False)
        else:
            type = None
        return AssignmentStmt(lvalues, expr, type)

    def parse_return_stmt(self) -> ReturnStmt:
        self.expect('return')
        expr = None
        current = self.current()
        if current.string == 'yield':
            self.parse_error()
        if not isinstance(current, Break):
            expr = self.parse_expression()
        node = ReturnStmt(expr)
        return node

    def parse_raise_stmt(self) -> RaiseStmt:
        self.expect('raise')
        expr = None
        from_expr = None
        if not isinstance(self.current(), Break):
            expr = self.parse_expression()
            if self.current_str() == 'from':
                self.expect('from')
                from_expr = self.parse_expression()
        node = RaiseStmt(expr, from_expr)
        return node

    def parse_assert_stmt(self) -> AssertStmt:
        self.expect('assert')
        expr = self.parse_expression()
        node = AssertStmt(expr)
        return node

    def parse_yield_or_yield_from_expr(self) -> Union[YieldFromExpr, YieldExpr]:
        self.expect("yield")
        expr = None
        node = YieldExpr(expr)  # type: Union[YieldFromExpr, YieldExpr]
        if not isinstance(self.current(), Break):
            if self.current_str() == "from":
                self.expect("from")
                expr = self.parse_expression()  # when yield from is assigned to a variable
                node = YieldFromExpr(expr)
            else:
                if self.current_str() == ')':
                    node = YieldExpr(None)
                else:
                    expr = self.parse_expression()
                    node = YieldExpr(expr)
        return node

    def parse_ellipsis(self) -> EllipsisExpr:
        self.expect('...')
        node = EllipsisExpr()
        return node

    def parse_del_stmt(self) -> DelStmt:
        self.expect('del')
        expr = self.parse_expression()
        node = DelStmt(expr)
        return node

    def parse_break_stmt(self) -> BreakStmt:
        self.expect('break')
        node = BreakStmt()
        return node

    def parse_continue_stmt(self) -> ContinueStmt:
        self.expect('continue')
        node = ContinueStmt()
        return node

    def parse_pass_stmt(self) -> PassStmt:
        self.expect('pass')
        node = PassStmt()
        return node

    def parse_global_decl(self) -> GlobalDecl:
        self.expect('global')
        names = self.parse_identifier_list()
        node = GlobalDecl(names)
        return node

    def parse_nonlocal_decl(self) -> NonlocalDecl:
        self.expect('nonlocal')
        names = self.parse_identifier_list()
        node = NonlocalDecl(names)
        return node

    def parse_identifier_list(self) -> List[str]:
        names = []
        while True:
            n = self.expect_type(Name)
            names.append(n.string)
            if self.current_str() != ',':
                break
            self.skip()
        return names

    def parse_while_stmt(self) -> WhileStmt:
        is_error = False
        self.expect('while')
        try:
            expr = self.parse_expression()
        except ParseError:
            is_error = True
        body, _ = self.parse_block()
        if self.current_str() == 'else':
            self.expect('else')
            else_body, _ = self.parse_block()
        else:
            else_body = None
        if is_error is not None:
            node = WhileStmt(expr, body, else_body)
            return node
        else:
            return None

    def parse_for_stmt(self) -> ForStmt:
        self.expect('for')
        index = self.parse_for_index_variables()
        self.expect('in')
        expr = self.parse_expression()

        body, _ = self.parse_block()

        if self.current_str() == 'else':
            self.expect('else')
            else_body, _ = self.parse_block()
        else:
            else_body = None

        node = ForStmt(index, expr, body, else_body)
        return node

    def parse_for_index_variables(self) -> Node:
        # Parse index variables of a 'for' statement.
        index_items = []
        force_tuple = False

        while True:
            v = self.parse_expression(precedence['in'],
                                      star_expr_allowed=True)  # Prevent parsing of for stmt 'in'
            index_items.append(v)
            if self.current_str() != ',':
                break
            self.skip()
            if self.current_str() == 'in':
                force_tuple = True
                break

        if len(index_items) == 1 and not force_tuple:
            index = index_items[0]
        else:
            index = TupleExpr(index_items)
            index.set_line(index_items[0].get_line())

        return index

    def parse_if_stmt(self) -> IfStmt:
        is_error = False

        self.expect('if')
        expr = []
        try:
            expr.append(self.parse_expression())
        except ParseError:
            is_error = True

        body = [self.parse_block()[0]]

        while self.current_str() == 'elif':
            self.expect('elif')
            try:
                expr.append(self.parse_expression())
            except ParseError:
                is_error = True
            body.append(self.parse_block()[0])

        if self.current_str() == 'else':
            self.expect('else')
            else_body, _ = self.parse_block()
        else:
            else_body = None

        if not is_error:
            node = IfStmt(expr, body, else_body)
            return node
        else:
            return None

    def parse_try_stmt(self) -> Node:
        self.expect('try')
        body, _ = self.parse_block()
        is_error = False
        vars = []  # type: List[NameExpr]
        types = []  # type: List[Node]
        handlers = []  # type: List[Block]
        while self.current_str() == 'except':
            self.expect('except')
            if not isinstance(self.current(), Colon):
                try:
                    t = self.current()
                    types.append(self.parse_expression(precedence[',']).set_line(t))
                    if self.current_str() == 'as':
                        self.expect('as')
                        vars.append(self.parse_name_expr())
                    elif self.pyversion[0] == 2 and self.current_str() == ',':
                        self.expect(',')
                        vars.append(self.parse_name_expr())
                    else:
                        vars.append(None)
                except ParseError:
                    is_error = True
            else:
                types.append(None)
                vars.append(None)
            handlers.append(self.parse_block()[0])
        if not is_error:
            if self.current_str() == 'else':
                self.skip()
                else_body, _ = self.parse_block()
            else:
                else_body = None
            if self.current_str() == 'finally':
                self.expect('finally')
                finally_body, _ = self.parse_block()
            else:
                finally_body = None
            node = TryStmt(body, vars, types, handlers, else_body,
                           finally_body)
            return node
        else:
            return None

    def parse_with_stmt(self) -> WithStmt:
        self.expect('with')
        exprs = []
        targets = []
        while True:
            expr = self.parse_expression(precedence[','])
            if self.current_str() == 'as':
                self.expect('as')
                target = self.parse_expression(precedence[','])
            else:
                target = None
            exprs.append(expr)
            targets.append(target)
            if self.current_str() != ',':
                break
            self.expect(',')
        body, _ = self.parse_block()
        return WithStmt(exprs, targets, body)

    def parse_print_stmt(self) -> PrintStmt:
        self.expect('print')
        args = []
        target = None
        if self.current_str() == '>>':
            self.skip()
            target = self.parse_expression(precedence[','])
            if self.current_str() == ',':
                self.skip()
                if isinstance(self.current(), Break):
                    self.parse_error()
            else:
                if not isinstance(self.current(), Break):
                    self.parse_error()
        comma = False
        while not isinstance(self.current(), Break):
            args.append(self.parse_expression(precedence[',']))
            if self.current_str() == ',':
                comma = True
                self.skip()
            else:
                comma = False
                break
        return PrintStmt(args, newline=not comma, target=target)

    def parse_exec_stmt(self) -> ExecStmt:
        self.expect('exec')
        expr = self.parse_expression(precedence['in'])
        variables1 = None
        variables2 = None
        if self.current_str() == 'in':
            self.skip()
            variables1 = self.parse_expression(precedence[','])
            if self.current_str() == ',':
                self.skip()
                variables2 = self.parse_expression(precedence[','])
        return ExecStmt(expr, variables1, variables2)

    # Parsing expressions

    def parse_expression(self, prec: int = 0, star_expr_allowed: bool = False) -> Node:
        """Parse a subexpression within a specific precedence context."""
        expr = None  # type: Node
        current = self.current()  # Remember token for setting the line number.

        # Parse a "value" expression or unary operator expression and store
        # that in expr.
        s = self.current_str()
        if s == '(':
            # Parerenthesised expression or cast.
            expr = self.parse_parentheses()
        elif s == '[':
            expr = self.parse_list_expr()
        elif s in ['-', '+', 'not', '~']:
            # Unary operation.
            expr = self.parse_unary_expr()
        elif s == 'lambda':
            expr = self.parse_lambda_expr()
        elif s == '{':
            expr = self.parse_dict_or_set_expr()
        elif s == '*' and star_expr_allowed:
            expr = self.parse_star_expr()
        elif s == '`' and self.pyversion[0] == 2:
            expr = self.parse_backquote_expr()
        else:
            if isinstance(current, Name):
                # Name expression.
                expr = self.parse_name_expr()
            elif isinstance(current, IntLit):
                expr = self.parse_int_expr()
            elif isinstance(current, StrLit):
                expr = self.parse_str_expr()
            elif isinstance(current, BytesLit):
                expr = self.parse_bytes_literal()
            elif isinstance(current, UnicodeLit):
                expr = self.parse_unicode_literal()
            elif isinstance(current, FloatLit):
                expr = self.parse_float_expr()
            elif isinstance(current, ComplexLit):
                expr = self.parse_complex_expr()
            elif isinstance(current, Keyword) and s == "yield":
                # The expression yield from and yield to assign
                expr = self.parse_yield_or_yield_from_expr()
            elif isinstance(current, EllipsisToken) and (self.pyversion[0] >= 3
                                                         or self.is_stub_file):
                expr = self.parse_ellipsis()
            else:
                # Invalid expression.
                self.parse_error()

        # Set the line of the expression node, if not specified. This
        # simplifies recording the line number as not every node type needs to
        # deal with it separately.
        if expr.line < 0:
            expr.set_line(current)

        # Parse operations that require a left argument (stored in expr).
        while True:
            current = self.current()
            s = self.current_str()
            if s == '(':
                # Call expression.
                expr = self.parse_call_expr(expr)
            elif s == '.':
                # Member access expression.
                expr = self.parse_member_expr(expr)
            elif s == '[':
                # Indexing expression.
                expr = self.parse_index_expr(expr)
            elif s == ',':
                # The comma operator is used to build tuples. Comma also
                # separates array items and function arguments, but in this
                # case the precedence is too low to build a tuple.
                if precedence[','] > prec:
                    expr = self.parse_tuple_expr(expr)
                else:
                    break
            elif s == 'for':
                if precedence['<for>'] > prec:
                    # List comprehension or generator expression. Parse as
                    # generator expression; it will be converted to list
                    # comprehension if needed elsewhere.
                    expr = self.parse_generator_expr(expr)
                else:
                    break
            elif s == 'if':
                # Conditional expression.
                if precedence['<if>'] > prec:
                    expr = self.parse_conditional_expr(expr)
                else:
                    break
            else:
                # Binary operation or a special case.
                if isinstance(current, Op):
                    op = self.current_str()
                    op_prec = precedence[op]
                    if op == 'not':
                        # Either "not in" or an error.
                        op_prec = precedence['in']
                    if op_prec > prec:
                        if op in op_comp:
                            expr = self.parse_comparison_expr(expr, op_prec)
                        else:
                            expr = self.parse_bin_op_expr(expr, op_prec)
                    else:
                        # The operation cannot be associated with the
                        # current left operand due to the precedence
                        # context; let the caller handle it.
                        break
                else:
                    # Not an operation that accepts a left argument; let the
                    # caller handle the rest.
                    break

            # Set the line of the expression node, if not specified. This
            # simplifies recording the line number as not every node type
            # needs to deal with it separately.
            if expr.line < 0:
                expr.set_line(current)

        return expr

    def parse_parentheses(self) -> Node:
        self.skip()
        if self.current_str() == ')':
            # Empty tuple ().
            expr = self.parse_empty_tuple_expr()  # type: Node
        else:
            # Parenthesised expression.
            expr = self.parse_expression(0, star_expr_allowed=True)
            self.expect(')')
        return expr

    def parse_star_expr(self) -> Node:
        star = self.expect('*')
        expr = self.parse_expression(precedence['*u'])
        expr = StarExpr(expr)
        if expr.line < 0:
            expr.set_line(star)
        return expr

    def parse_empty_tuple_expr(self) -> TupleExpr:
        self.expect(')')
        node = TupleExpr([])
        return node

    def parse_list_expr(self) -> Node:
        """Parse list literal or list comprehension."""
        items = []
        self.expect('[')
        while self.current_str() != ']' and not self.eol():
            items.append(self.parse_expression(precedence['<for>'], star_expr_allowed=True))
            if self.current_str() != ',':
                break
            self.expect(',')
        if self.current_str() == 'for' and len(items) == 1:
            items[0] = self.parse_generator_expr(items[0])
        self.expect(']')
        if len(items) == 1 and isinstance(items[0], GeneratorExpr):
            return ListComprehension(cast(GeneratorExpr, items[0]))
        else:
            expr = ListExpr(items)
            return expr

    def parse_generator_expr(self, left_expr: Node) -> GeneratorExpr:
        tok = self.current()
        indices, sequences, condlists = self.parse_comp_for()

        gen = GeneratorExpr(left_expr, indices, sequences, condlists)
        gen.set_line(tok)
        return gen

    def parse_comp_for(self) -> Tuple[List[Node], List[Node], List[List[Node]]]:
        indices = []
        sequences = []
        condlists = []  # type: List[List[Node]]
        while self.current_str() == 'for':
            conds = []
            self.expect('for')
            index = self.parse_for_index_variables()
            indices.append(index)
            self.expect('in')
            if self.pyversion[0] >= 3:
                sequence = self.parse_expression(precedence['<if>'])
            else:
                sequence = self.parse_expression_list()
            sequences.append(sequence)
            while self.current_str() == 'if':
                self.skip()
                conds.append(self.parse_expression(precedence['<if>']))
            condlists.append(conds)

        return indices, sequences, condlists

    def parse_expression_list(self) -> Node:
        prec = precedence['<if>']
        expr = self.parse_expression(prec)
        if self.current_str() != ',':
            return expr
        else:
            t = self.current()
            return self.parse_tuple_expr(expr, prec).set_line(t)

    def parse_conditional_expr(self, left_expr: Node) -> ConditionalExpr:
        self.expect('if')
        cond = self.parse_expression(precedence['<if>'])
        self.expect('else')
        else_expr = self.parse_expression(precedence['<if>'])
        return ConditionalExpr(cond, left_expr, else_expr)

    def parse_dict_or_set_expr(self) -> Node:
        items = []  # type: List[Tuple[Node, Node]]
        self.expect('{')
        while self.current_str() != '}' and not self.eol():
            key = self.parse_expression(precedence['<for>'])
            if self.current_str() in [',', '}'] and items == []:
                return self.parse_set_expr(key)
            elif self.current_str() == 'for' and items == []:
                return self.parse_set_comprehension(key)
            elif self.current_str() != ':':
                self.parse_error()
            colon = self.expect(':')
            value = self.parse_expression(precedence['<for>'])
            if self.current_str() == 'for' and items == []:
                return self.parse_dict_comprehension(key, value, colon)
            items.append((key, value))
            if self.current_str() != ',':
                break
            self.expect(',')
        self.expect('}')
        node = DictExpr(items)
        return node

    def parse_set_expr(self, first: Node) -> SetExpr:
        items = [first]
        while self.current_str() != '}' and not self.eol():
            self.expect(',')
            if self.current_str() == '}':
                break
            items.append(self.parse_expression(precedence[',']))
        self.expect('}')
        expr = SetExpr(items)
        return expr

    def parse_set_comprehension(self, expr: Node):
        gen = self.parse_generator_expr(expr)
        self.expect('}')
        set_comp = SetComprehension(gen)
        return set_comp

    def parse_dict_comprehension(self, key: Node, value: Node,
                                 colon: Token) -> DictionaryComprehension:
        indices, sequences, condlists = self.parse_comp_for()
        dic = DictionaryComprehension(key, value, indices, sequences, condlists)
        dic.set_line(colon)
        self.expect('}')
        return dic

    def parse_tuple_expr(self, expr: Node,
                         prec: int = precedence[',']) -> TupleExpr:
        items = [expr]
        while True:
            self.expect(',')
            if (self.current_str() in [')', ']', '=', ':'] or
                    isinstance(self.current(), Break)):
                break
            items.append(self.parse_expression(prec, star_expr_allowed=True))
            if self.current_str() != ',': break
        node = TupleExpr(items)
        return node

    def parse_backquote_expr(self) -> BackquoteExpr:
        self.expect('`')
        expr = self.parse_expression()
        self.expect('`')
        return BackquoteExpr(expr)

    def parse_name_expr(self) -> NameExpr:
        tok = self.expect_type(Name)
        node = NameExpr(tok.string)
        node.set_line(tok)
        return node

    octal_int = re.compile('0+[1-9]')

    def parse_int_expr(self) -> IntExpr:
        tok = self.expect_type(IntLit)
        string = tok.string.rstrip('lL')  # Strip L prefix (Python 2 long literals)
        if self.octal_int.match(string):
            value = int(string, 8)
        else:
            value = int(string, 0)
        node = IntExpr(value)
        return node

    def parse_str_expr(self) -> Node:
        # XXX \uxxxx literals
        token = self.expect_type(StrLit)
        value = cast(StrLit, token).parsed()
        is_unicode = False
        while isinstance(self.current(), (StrLit, UnicodeLit)):
            token = self.skip()
            if isinstance(token, StrLit):
                value += token.parsed()
            elif isinstance(token, UnicodeLit):
                value += token.parsed()
                is_unicode = True
        if is_unicode or (self.pyversion[0] == 2 and 'unicode_literals' in self.future_options):
            node = UnicodeExpr(value)  # type: Node
        else:
            node = StrExpr(value)
        return node

    def parse_bytes_literal(self) -> Node:
        # XXX \uxxxx literals
        tok = [self.expect_type(BytesLit)]
        value = (cast(BytesLit, tok[0])).parsed()
        while isinstance(self.current(), BytesLit):
            t = cast(BytesLit, self.skip())
            value += t.parsed()
        if self.pyversion[0] >= 3:
            node = BytesExpr(value)  # type: Node
        else:
            node = StrExpr(value)
        return node

    def parse_unicode_literal(self) -> Node:
        # XXX \uxxxx literals
        token = self.expect_type(UnicodeLit)
        value = cast(UnicodeLit, token).parsed()
        while isinstance(self.current(), (UnicodeLit, StrLit)):
            token = cast(Union[UnicodeLit, StrLit], self.skip())
            value += token.parsed()
        if self.pyversion[0] >= 3:
            # Python 3.3 supports u'...' as an alias of '...'.
            node = StrExpr(value)  # type: Node
        else:
            node = UnicodeExpr(value)
        return node

    def parse_float_expr(self) -> FloatExpr:
        tok = self.expect_type(FloatLit)
        node = FloatExpr(float(tok.string))
        return node

    def parse_complex_expr(self) -> ComplexExpr:
        tok = self.expect_type(ComplexLit)
        node = ComplexExpr(complex(tok.string))
        return node

    def parse_call_expr(self, callee: Any) -> CallExpr:
        self.expect('(')
        args, kinds, names = self.parse_arg_expr()
        self.expect(')')
        node = CallExpr(callee, args, kinds, names)
        return node

    def parse_arg_expr(self) -> Tuple[List[Node], List[int], List[str]]:
        """Parse arguments in a call expression (within '(' and ')').

        Return a tuple with these items:
          argument expressions
          argument kinds
          argument names (for named arguments; None for ordinary args)
        """
        args = []   # type: List[Node]
        kinds = []  # type: List[int]
        names = []  # type: List[str]
        var_arg = False
        dict_arg = False
        named_args = False
        while self.current_str() != ')' and not self.eol() and not dict_arg:
            if isinstance(self.current(), Name) and self.peek().string == '=':
                # Named argument
                name = self.expect_type(Name)
                self.expect('=')
                kinds.append(nodes.ARG_NAMED)
                names.append(name.string)
                named_args = True
            elif (self.current_str() == '*' and not var_arg and not dict_arg):
                # *args
                var_arg = True
                self.expect('*')
                kinds.append(nodes.ARG_STAR)
                names.append(None)
            elif self.current_str() == '**':
                # **kwargs
                self.expect('**')
                dict_arg = True
                kinds.append(nodes.ARG_STAR2)
                names.append(None)
            elif not var_arg and not named_args:
                # Ordinary argument
                kinds.append(nodes.ARG_POS)
                names.append(None)
            else:
                self.parse_error()
            args.append(self.parse_expression(precedence[',']))
            if self.current_str() != ',':
                break
            self.expect(',')
        return args, kinds, names

    def parse_member_expr(self, expr: Any) -> Node:
        self.expect('.')
        name = self.expect_type(Name)
        if (isinstance(expr, CallExpr) and isinstance(expr.callee, NameExpr)
                and cast(NameExpr, expr.callee).name == 'super'):
            # super() expression
            node = SuperExpr(name.string)  # type: Node
        else:
            node = MemberExpr(expr, name.string)
        return node

    def parse_index_expr(self, base: Any) -> IndexExpr:
        self.expect('[')
        index = self.parse_slice_item()
        if self.current_str() == ',':
            # Extended slicing such as x[1:, :2].
            items = [index]
            while self.current_str() == ',':
                self.skip()
                if self.current_str() == ']' or isinstance(self.current(), Break):
                    break
                items.append(self.parse_slice_item())
            index = TupleExpr(items)
            index.set_line(items[0].line)
        self.expect(']')
        node = IndexExpr(base, index)
        return node

    def parse_slice_item(self) -> Node:
        if self.current_str() != ':':
            if self.current_str() == '...':
                # Ellipsis is valid here even in Python 2 (but not elsewhere).
                ellipsis = EllipsisExpr()
                token = self.skip()
                ellipsis.set_line(token)
                return ellipsis
            else:
                item = self.parse_expression(precedence[','])
        else:
            item = None
        if self.current_str() == ':':
            # Slice.
            index = item
            colon = self.expect(':')
            if self.current_str() not in (']', ':', ','):
                end_index = self.parse_expression(precedence[','])
            else:
                end_index = None
            stride = None
            if self.current_str() == ':':
                self.expect(':')
                if self.current_str() not in (']', ','):
                    stride = self.parse_expression(precedence[','])
            item = SliceExpr(index, end_index, stride).set_line(colon.line)
        return item

    def parse_bin_op_expr(self, left: Node, prec: int) -> OpExpr:
        op = self.expect_type(Op)
        op_str = op.string
        if op_str == '~':
            self.ind -= 1
            self.parse_error()
        right = self.parse_expression(prec)
        node = OpExpr(op_str, left, right)
        return node

    def parse_comparison_expr(self, left: Node, prec: int) -> ComparisonExpr:
        operators_str = []
        operands = [left]

        while True:
            op = self.expect_type(Op)
            op_str = op.string
            if op_str == 'not':
                if self.current_str() == 'in':
                    op_str = 'not in'
                    self.skip()
                else:
                    self.parse_error()
            elif op_str == 'is' and self.current_str() == 'not':
                op_str = 'is not'
                self.skip()

            operators_str.append(op_str)
            operand = self.parse_expression(prec)
            operands.append(operand)

            # Continue if next token is a comparison operator
            self.current()
            s = self.current_str()
            if s not in op_comp:
                break

        node = ComparisonExpr(operators_str, operands)
        return node

    def parse_unary_expr(self) -> UnaryExpr:
        op_tok = self.skip()
        op = op_tok.string
        if op == '-' or op == '+':
            prec = precedence['-u']
        else:
            prec = precedence[op]
        expr = self.parse_expression(prec)
        node = UnaryExpr(op, expr)
        return node

    def parse_lambda_expr(self) -> FuncExpr:
        lambda_tok = self.expect('lambda')

        args, extra_stmts = self.parse_arg_list(allow_signature=False)

        # Use 'object' as the placeholder return type; it will be inferred
        # later. We can't use 'Any' since it could make type inference results
        # less precise.
        ret_type = UnboundType('__builtins__.object')
        typ = self.build_func_annotation(ret_type, args,
                                         lambda_tok.line, is_default_ret=True)

        colon = self.expect(':')

        expr = self.parse_expression(precedence[','])

        nodes = [ReturnStmt(expr).set_line(lambda_tok)]
        # Potentially insert extra assignment statements to the beginning of the
        # body, used to decompose Python 2 tuple arguments.
        nodes[:0] = extra_stmts
        body = Block(nodes)
        body.set_line(colon)

        return FuncExpr(args, body, typ)

    # Helper methods

    def skip(self) -> Token:
        self.ind += 1
        return self.tok[self.ind - 1]

    def expect(self, string: str) -> Token:
        if self.current_str() == string:
            self.ind += 1
            return self.tok[self.ind - 1]
        else:
            self.parse_error()

    def expect_indent(self) -> Token:
        if isinstance(self.current(), Indent):
            return self.expect_type(Indent)
        else:
            self.fail('Expected an indented block', self.current().line)
            return none

    def fail(self, msg: str, line: int) -> None:
        self.errors.report(line, msg)

    def expect_type(self, typ: type) -> Token:
        current = self.current()
        if isinstance(current, typ):
            self.ind += 1
            return current
        else:
            self.parse_error()

    def expect_colon_and_break(self) -> Tuple[Token, Token]:
        return self.expect_type(Colon), self.expect_type(Break)

    def expect_break(self) -> Token:
        return self.expect_type(Break)

    def current(self) -> Token:
        return self.tok[self.ind]

    def current_str(self) -> str:
        return self.current().string

    def peek(self) -> Token:
        return self.tok[self.ind + 1]

    def parse_error(self) -> None:
        self.parse_error_at(self.current())
        raise ParseError()

    def parse_error_at(self, tok: Token, skip: bool = True, reason: Optional[str] = None) -> None:
        msg = ''
        if isinstance(tok, LexError):
            msg = token_repr(tok)
            msg = msg[0].upper() + msg[1:]
        elif isinstance(tok, Indent) or isinstance(tok, Dedent):
            msg = 'Inconsistent indentation'
        else:
            formatted_reason = ": {}".format(reason) if reason else ""
            msg = 'Parse error before {}{}'.format(token_repr(tok), formatted_reason)

        self.errors.report(tok.line, msg)

        if skip:
            self.skip_until_next_line()

    def skip_until_break(self) -> None:
        n = 0
        while (not isinstance(self.current(), Break)
               and not isinstance(self.current(), Eof)):
            self.skip()
            n += 1
        if isinstance(self.tok[self.ind - 1], Colon) and n > 1:
            self.ind -= 1

    def skip_until_next_line(self) -> None:
        self.skip_until_break()
        if isinstance(self.current(), Break):
            self.skip()

    def eol(self) -> bool:
        return isinstance(self.current(), Break) or self.eof()

    def eof(self) -> bool:
        return isinstance(self.current(), Eof)

    # Type annotation related functionality

    def parse_type(self) -> Type:
        try:
            typ, self.ind = parse_type(self.tok, self.ind)
        except TypeParseError as e:
            self.parse_error_at(e.token, reason=e.message)
            raise ParseError()
        return typ

    annotation_prefix_re = re.compile(r'#\s*type:')
    ignore_prefix_re = re.compile(r'ignore\b')

    def parse_type_comment(self, token: Token, signature: bool) -> Type:
        """Parse a '# type: ...' annotation.

        Return None if no annotation found. If signature is True, expect
        a type signature of form (...) -> t.
        """
        whitespace_or_comments = token.rep().strip()
        if self.annotation_prefix_re.match(whitespace_or_comments):
            type_as_str = whitespace_or_comments.split(':', 1)[1].strip()
            if self.ignore_prefix_re.match(type_as_str):
                # Actually a "# type: ignore" annotation -> not a type.
                return None
            tokens = lex.lex(type_as_str, token.line)[0]
            if len(tokens) < 2:
                # Empty annotation (only Eof token)
                self.errors.report(token.line, 'Empty type annotation')
                return None
            try:
                if not signature:
                    type, index = parse_types(tokens, 0)
                else:
                    type, index = parse_signature(tokens)
            except TypeParseError as e:
                self.parse_error_at(e.token, skip=False, reason=e.message)
                return None
            if index < len(tokens) - 2:
                self.parse_error_at(tokens[index], skip=False)
                return None
            return type
        else:
            return None


class ParseError(Exception): pass


def token_repr(tok: Token) -> str:
    """Return a representation of a token for use in parse error messages."""
    if isinstance(tok, Break):
        return 'end of line'
    elif isinstance(tok, Eof):
        return 'end of file'
    elif isinstance(tok, Keyword) or isinstance(tok, Name):
        return '"{}"'.format(tok.string)
    elif isinstance(tok, IntLit) or isinstance(tok, FloatLit) or isinstance(tok, ComplexLit):
        return 'numeric literal'
    elif isinstance(tok, StrLit) or isinstance(tok, UnicodeLit):
        return 'string literal'
    elif (isinstance(tok, Punct) or isinstance(tok, Op)
          or isinstance(tok, Colon)):
        return tok.string
    elif isinstance(tok, Bom):
        return 'byte order mark'
    elif isinstance(tok, Indent):
        return 'indent'
    elif isinstance(tok, Dedent):
        return 'dedent'
    elif isinstance(tok, EllipsisToken):
        return '...'
    else:
        if isinstance(tok, LexError):
            t = tok.type
            if t == lex.NUMERIC_LITERAL_ERROR:
                return 'invalid numeric literal'
            elif t == lex.UNTERMINATED_STRING_LITERAL:
                return 'unterminated string literal'
            elif t == lex.INVALID_CHARACTER:
                msg = 'unrecognized character'
                if ord(tok.string) in range(33, 127):
                    msg += ' ' + tok.string
                return msg
            elif t == lex.INVALID_DEDENT:
                return 'inconsistent indentation'
            elif t == lex.DECODE_ERROR:
                return tok.message
        raise ValueError('Unknown token {}'.format(repr(tok)))


if __name__ == '__main__':
    # Parse a file and dump the AST (or display errors).
    import sys

    def usage():
        print('Usage: parse.py [--py2] [--quiet] FILE [...]', file=sys.stderr)
        sys.exit(2)

    args = sys.argv[1:]
    pyversion = defaults.PYTHON3_VERSION
    quiet = False
    while args and args[0].startswith('--'):
        if args[0] == '--py2':
            pyversion = defaults.PYTHON2_VERSION
        elif args[0] == '--quiet':
            quiet = True
        else:
            usage()
        args = args[1:]
    if len(args) < 1:
        usage()
    status = 0
    for fnam in args:
        s = open(fnam, 'rb').read()
        errors = Errors()
        try:
            tree = parse(s, fnam, pyversion=pyversion)
            if not quiet:
                print(tree)
        except CompileError as e:
            for msg in e.messages:
                sys.stderr.write('%s\n' % msg)
            status = 1
    sys.exit(status)
